@@ -1,6 +1,17 @@
 #ifndef ROFI_HELPER_H
 #define ROFI_HELPER_H
 #include "rofi.h"
+
+#include <pango/pango.h>
+/**
+ * @defgroup HELPERS Helpers
+ */
+/**
+ * @defgroup HELPER Helper
+ * @ingroup HELPERS
+ *
+ * @{
+ */
 /**
  * @param string The input string.
  * @param output Pointer to 2 dimensional array with parsed string.
@@ -13,23 +24,6 @@
  */
 int helper_parse_setup ( char * string, char ***output, int *length, ... );
 
-/**
- * Implementation of fgets with custom separator.
- */
-char* fgets_s ( char* s, unsigned int n, FILE *iop, char sep );
-
-/**
- * @param token The string for which we want a collation key.
- * @param case_sensitive Whether case is significant.
- *
- * Get a collation key for @p token. @p token must be a null-terminated string.
- * This collation key can be used for matching the user input against the list
- * of commands, windows, or ssh commands.
- *
- * @returns A newly allocated string containing the collation key.
- */
-char *token_collate_key ( const char *token, int case_sensitive );
-
 /**
  * @param input The input string.
  * @param case_sensitive Whether case is significant.
@@ -38,7 +32,8 @@ char *token_collate_key ( const char *token, int case_sensitive );
  *
  * @returns a newly allocated 2 dimensional array of strings.
  */
-char **tokenize ( const char *input, int case_sensitive );
+GRegex **tokenize ( const char *input, int case_sensitive );
+void tokenize_free ( GRegex ** tokens );
 
 /**
  * @param key The key to search for
@@ -91,20 +86,14 @@ int find_arg_str ( const char * const key, char** val );
 int find_arg ( const char * const key );
 
 /**
- * @params tokens
  * @param tokens  List of (input) tokens to match.
  * @param input   The entry to match against.
- * @param case_sensitive Whether case is significant.
- * @param index   The current selected index.
- * @param data    User data.
  *
  * Tokenized match, match tokens to line input.
  *
- * @returns 1 when matches, 0 otherwise
+ * @returns TRUE when matches, FALSE otherwise
  */
-int token_match ( char **tokens, const char *input, int not_ascii, int case_sensitive,
-                  __attribute__( ( unused ) ) unsigned int index,
-                  __attribute__( ( unused ) ) Switcher * data );
+int token_match ( GRegex **tokens, const char *input );
 
 /**
  * @param cmd The command to execute.
@@ -134,7 +123,7 @@ void remove_pid_file ( int fd );
  *
  * This functions exits the program with 1 when it finds an invalid configuration.
  */
-void config_sanity_check ( Display *display );
+int config_sanity_check ( void );
 
 /**
  * @param arg string to parse.
@@ -154,9 +143,20 @@ char helper_parse_char ( const char *arg );
 void cmd_set_arguments ( int argc, char **argv );
 
 /**
- * @param str a UTF8 string
- * @return 1 if the string contains any non-ascii codepoints
+ * @param input The path to expand
+ *
+ * Expand path, both `~` and `~<user>`
+ *
+ * @returns path
  */
-int is_not_ascii ( const char *str );
+char *rofi_expand_path ( const char *input );
+unsigned int levenshtein ( const char *needle, const char *haystack );
 
+/**
+ * Convert string to valid utf-8, replacing invalid parts with replacement character.
+ */
+char * rofi_force_utf8 ( gchar *data, ssize_t length );
+char * rofi_latin_to_utf8_strdup ( const char *input, gssize length );
+PangoAttrList *token_match_get_pango_attr ( GRegex **tokens, const char *input, PangoAttrList *retv );
+/*@}*/
 #endif // ROFI_HELPER_H
