@@ -1,24 +1,48 @@
 #ifndef ROFI_KEYB_H
 #define ROFI_KEYB_H
 
-typedef enum _KeyBindingAction
+/**
+ * @defgroup KEYB KeyboardBindings
+ *
+ * @{
+ */
+
+typedef enum
 {
+    /** Paste from primary clipboard */
     PASTE_PRIMARY = 0,
+    /** Paste from secondary clipboard */
     PASTE_SECONDARY,
+    /** Clear the entry box. */
     CLEAR_LINE,
+    /** Move to front of text */
     MOVE_FRONT,
+    /** Move to end of text */
     MOVE_END,
+    /** Move on word back */
     MOVE_WORD_BACK,
+    /** Move on word forward */
     MOVE_WORD_FORWARD,
+    /** Move character back */
     MOVE_CHAR_BACK,
+    /** Move character forward */
     MOVE_CHAR_FORWARD,
+    /** Remove previous word */
     REMOVE_WORD_BACK,
+    /** Remove next work */
     REMOVE_WORD_FORWARD,
+    /** Remove next character */
     REMOVE_CHAR_FORWARD,
+    /** Remove previous character */
     REMOVE_CHAR_BACK,
+    /** Remove till EOL */
+    REMOVE_TO_EOL,
+    /** Remove till SOL */
+    REMOVE_TO_SOL,
+    /** Accept the current selected entry */
     ACCEPT_ENTRY,
+    ACCEPT_ALT,
     ACCEPT_CUSTOM,
-    ACCEPT_ENTRY_CONTINUE,
     MODE_NEXT,
     MODE_PREVIOUS,
     TOGGLE_CASE_SENSITIVITY,
@@ -53,6 +77,8 @@ typedef enum _KeyBindingAction
     CUSTOM_17,
     CUSTOM_18,
     CUSTOM_19,
+    SCREENSHOT,
+    TOGGLE_SORT,
     NUM_ABE
 } KeyBindingAction;
 
@@ -60,7 +86,7 @@ typedef enum _KeyBindingAction
  * Parse the keybindings.
  * This should be called after the setting system is initialized.
  */
-void parse_keys_abe ( void );
+gboolean parse_keys_abe ( void );
 
 /**
  * Setup the keybindings
@@ -74,8 +100,12 @@ void setup_abe ( void );
 void cleanup_abe ( void );
 
 /**
- * Check if this key has been triggered.
- * @returns TRUE if key combo matches, FALSE otherwise.
+ * Find if a binding has been triggered.
+ * @returns NUM_ABE if no key combo matches, a valid action otherwise.
  */
-int abe_test_action ( KeyBindingAction action, unsigned int mask, KeySym key );
+KeyBindingAction abe_find_action ( unsigned int mask, xkb_keysym_t key );
+
+void abe_trigger_release ( void );
+
+/*@}*/
 #endif // ROFI_KEYB_H
