@@ -1,25 +1,29 @@
 #ifndef ROFI_TEXTBOX_H
 #define ROFI_TEXTBOX_H
 
-#include <X11/Xutil.h>
+#include <xkbcommon/xkbcommon.h>
 #include <pango/pango.h>
 #include <pango/pango-fontmap.h>
+#include <pango/pangocairo.h>
 #include <cairo.h>
+#include "widget.h"
+#include "x11-helper.h"
+#include "keyb.h"
+
+/**
+ * @defgroup Textbox Textbox
+ * @ingroup Widgets
+ *
+ * @{
+ */
 
 typedef struct
 {
-    double red, green, blue, alpha;
-} Color;
-
-typedef struct
-{
+    Widget          widget;
     unsigned long   flags;
-    short           x, y, w, h;
     short           cursor;
     Color           color_fg, color_bg;
     char            *text;
-    XIM             xim;
-    XIC             xic;
     PangoLayout     *layout;
     int             tbft;
     int             markup;
@@ -29,18 +33,22 @@ typedef struct
     cairo_t         *main_draw;
 
     int             update;
+    int             blink;
+    guint           blink_timeout;
 } textbox;
 
 typedef enum
 {
-    TB_AUTOHEIGHT    = 1 << 0,
-        TB_AUTOWIDTH = 1 << 1,
-        TB_LEFT      = 1 << 16,
-        TB_RIGHT     = 1 << 17,
-        TB_CENTER    = 1 << 18,
-        TB_EDITABLE  = 1 << 19,
-        TB_MARKUP    = 1 << 20,
-        TB_WRAP      = 1 << 21,
+    TB_AUTOHEIGHT = 1 << 0,
+    TB_AUTOWIDTH  = 1 << 1,
+    TB_LEFT       = 1 << 16,
+    TB_RIGHT      = 1 << 17,
+    TB_CENTER     = 1 << 18,
+    TB_EDITABLE   = 1 << 19,
+    TB_MARKUP     = 1 << 20,
+    TB_WRAP       = 1 << 21,
+    TB_PASSWORD   = 1 << 22,
+    TB_INDICATOR  = 1 << 23,
 } TextboxFlags;
 
 typedef enum
@@ -49,15 +57,16 @@ typedef enum
     NORMAL     = 0,
     URGENT     = 1,
     ACTIVE     = 2,
-    MARKUP     = 4,
+    SELECTED   = 4,
+    MARKUP     = 8,
 
     // Alternating row.
-    ALT        = 8,
+    ALT        = 16,
     // Render font highlighted (inverted colors.)
-    HIGHLIGHT  = 16,
+    HIGHLIGHT  = 32,
 
     FMOD_MASK  = ( ALT | HIGHLIGHT ),
-    STATE_MASK = ~( MARKUP | ALT | HIGHLIGHT )
+    STATE_MASK = ~( SELECTED | MARKUP | ALT | HIGHLIGHT )
 } TextBoxFontType;
 
 textbox* textbox_create ( TextboxFlags flags,
@@ -89,20 +98,22 @@ void textbox_text ( textbox *tb, const char *text );
 
 /**
  * @param tb  Handle to the textbox
+ * @param draw The cairo object used to draw itself.
  *
  * Render the textbox.
  */
 void textbox_draw ( textbox *tb, cairo_t *draw );
 
+int textbox_keybinding ( textbox *tb, KeyBindingAction action );
 /**
- * @param tb  Handle to the textbox
- * @param ev  XEvent key inputs to textbox
+ * @param tb Handle to the textbox
+ * @param pad The text to insert
+ * @param pad_len the length of the text
  *
- * Let the textbox handle the input event.
- *
- * @returns if the key was handled (1), unhandled(0) or handled and return was pressed (-1)
+ * The text should be one insert from a keypress..  the first gunichar is validated to be (or not) control
+ * return TRUE if inserted
  */
-int textbox_keypress ( textbox *tb, XIC xic, XEvent *ev );
+gboolean textbox_append_char ( textbox *tb, char *pad, int pad_len );
 
 /**
  * @param tb  Handle to the textbox
@@ -120,29 +131,21 @@ void textbox_cursor_end ( textbox *tb );
 void textbox_cursor ( textbox *tb, int pos );
 
 /**
- * @param tb  Handle to the textbox
- * @param x   The new x coordinate to move the window to
- * @param y   The new y coordinate to move the window to
- *
- * Move the window to x,y position.
- */
-void textbox_move ( textbox *tb, int x, int y );
-
-/**
- * @param tb  Handle to the textbox
- * @param pos The position to insert the string at
- * @param str The string to insert.
+ * @param tb   Handle to the textbox
+ * @param pos  The position to insert the string at
+ * @param str  The string to insert.
+ * @param slen The length of the string.
  *
  * Insert the string str at position pos.
  */
-void textbox_insert ( textbox *tb, int pos, char *str );
+void textbox_insert ( textbox *tb, int pos, char *str, int slen );
 
 /**
  * Setup the cached fonts. This is required to do
  * before any of the textbox_ functions is called.
  * Clean with textbox_cleanup()
  */
-void textbox_setup ( Display *display );
+void textbox_setup ( void );
 
 /**
  * Cleanup the allocated colors and fonts by textbox_setup().
@@ -231,4 +234,11 @@ void textbox_delete ( textbox *tb, int pos, int dlen );
 
 void textbox_moveresize ( textbox *tb, int x, int y, int w, int h );
 int textbox_get_estimated_char_height ( void );
+void textbox_set_pango_context ( PangoContext *p );
+void textbox_set_pango_attributes ( textbox *tb, PangoAttrList *list );
+
+PangoAttrList *textbox_get_pango_attributes ( textbox *tb );
+
+const char *textbox_get_visible_text ( textbox *tb );
+/*@}*/
 #endif //ROFI_TEXTBOX_H
