@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2015 Qball Cow <qball@gmpclient.org>
+ * Copyright 2013-2016 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -28,38 +28,39 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <rofi.h>
+#include "settings.h"
 
 #include <dialogs/dialogs.h>
 
 /**
- * Combi Switcher
+ * Combi Mode
  */
-typedef struct _CombiModePrivateData
+typedef struct
 {
     // List of (combined) entries.
-    char         **cmd_list;
     unsigned int cmd_list_length;
     // List to validate where each switcher starts.
     unsigned int *starts;
     unsigned int *lengths;
     // List of switchers to combine.
     unsigned int num_switchers;
-    Switcher     **switchers;
-    char         *cache;
+    Mode         **switchers;
+    Mode         *current;
 } CombiModePrivateData;
 
-static void combi_mode_parse_switchers ( Switcher *sw )
+static void combi_mode_parse_switchers ( Mode *sw )
 {
-    CombiModePrivateData *pd     = sw->private_data;
+    CombiModePrivateData *pd     = mode_get_private_data ( sw );
     char                 *savept = NULL;
     // Make a copy, as strtok will modify it.
     char                 *switcher_str = g_strdup ( config.combi_modi );
+    const char * const   sep           = ",";
     // Split token on ','. This modifies switcher_str.
-    for ( char *token = strtok_r ( switcher_str, ",", &savept ); token != NULL;
-          token = strtok_r ( NULL, ",", &savept ) ) {
+    for ( char *token = strtok_r ( switcher_str, sep, &savept ); token != NULL;
+          token = strtok_r ( NULL, sep, &savept ) ) {
         // Resize and add entry.
-        pd->switchers = (Switcher * *) g_realloc ( pd->switchers,
-                                                   sizeof ( Switcher* ) * ( pd->num_switchers + 1 ) );
+        pd->switchers = (Mode * *) g_realloc ( pd->switchers,
+                                               sizeof ( Mode* ) * ( pd->num_switchers + 1 ) );
 
         // Window switcher.
         #ifdef WINDOW_MODE
@@ -84,7 +85,7 @@ static void combi_mode_parse_switchers ( Switcher *sw )
         }
         else {
             // If not build in, use custom switchers.
-            Switcher *sw = script_switcher_parse_setup ( token );
+            Mode *sw = script_switcher_parse_setup ( token );
             if ( sw != NULL ) {
                 pd->switchers[pd->num_switchers++] = sw;
             }
@@ -100,69 +101,58 @@ static void combi_mode_parse_switchers ( Switcher *sw )
     g_free ( switcher_str );
 }
 
-static void combi_mode_init ( Switcher *sw )
+static int combi_mode_init ( Mode *sw )
 {
-    if ( sw->private_data == NULL ) {
+    if ( mode_get_private_data ( sw ) == NULL ) {
         CombiModePrivateData *pd = g_malloc0 ( sizeof ( *pd ) );
-        sw->private_data = (void *) pd;
+        mode_set_private_data ( sw, (void *) pd );
         combi_mode_parse_switchers ( sw );
         pd->starts  = g_malloc0 ( sizeof ( int ) * pd->num_switchers );
         pd->lengths = g_malloc0 ( sizeof ( int ) * pd->num_switchers );
         for ( unsigned int i = 0; i < pd->num_switchers; i++ ) {
-            pd->switchers[i]->init ( pd->switchers[i] );
+            if ( !mode_init ( pd->switchers[i] ) ) {
+                return FALSE;
+            }
         }
-    }
-}
-static char ** combi_mode_get_data ( unsigned int *length, Switcher *sw )
-{
-    CombiModePrivateData *pd = sw->private_data;
-    if ( pd->cmd_list == NULL ) {
-        pd->cmd_list_length = 0;
-        for ( unsigned int i = 0; i < pd->num_switchers; i++ ) {
-            unsigned int length = 0;
-            pd->switchers[i]->get_data ( &length, pd->switchers[i] );
-            pd->starts[i]        = pd->cmd_list_length;
-            pd->lengths[i]       = length;
-            pd->cmd_list_length += length;
-        }
-        // Fill the list.
-        pd->cmd_list = g_malloc0 ( ( pd->cmd_list_length + 1 ) * sizeof ( char* ) );
-        for ( unsigned i = 0; i < pd->num_switchers; i++ ) {
-            unsigned int length = 0;
-            char         **retv = pd->switchers[i]->get_data ( &length, pd->switchers[i] );
-            for ( unsigned int j = 0; j < length; j++ ) {
-                pd->cmd_list[j + pd->starts[i]] = retv[j];
+        if ( pd->cmd_list_length == 0 ) {
+            pd->cmd_list_length = 0;
+            for ( unsigned int i = 0; i < pd->num_switchers; i++ ) {
+                unsigned int length = mode_get_num_entries ( pd->switchers[i] );
+                pd->starts[i]        = pd->cmd_list_length;
+                pd->lengths[i]       = length;
+                pd->cmd_list_length += length;
             }
         }
     }
-    *length = pd->cmd_list_length;
-    return pd->cmd_list;
+    return TRUE;
 }
-static void combi_mode_destroy ( Switcher *sw )
+static unsigned int combi_mode_get_num_entries ( const Mode *sw )
 {
-    CombiModePrivateData *pd = (CombiModePrivateData *) sw->private_data;
+    const CombiModePrivateData *pd = (const CombiModePrivateData *) mode_get_private_data ( sw );
+    return pd->cmd_list_length;
+}
+static void combi_mode_destroy ( Mode *sw )
+{
+    CombiModePrivateData *pd = (CombiModePrivateData *) mode_get_private_data ( sw );
     if ( pd != NULL ) {
-        g_free ( pd->cmd_list );
         g_free ( pd->starts );
         g_free ( pd->lengths );
         // Cleanup switchers.
         for ( unsigned int i = 0; i < pd->num_switchers; i++ ) {
-            pd->switchers[i]->destroy ( pd->switchers[i] );
+            mode_destroy ( pd->switchers[i] );
         }
         g_free ( pd->switchers );
-        g_free ( pd->cache );
         g_free ( pd );
-        sw->private_data = NULL;
+        mode_set_private_data ( sw, NULL );
     }
 }
-static SwitcherMode combi_mode_result ( int mretv, char **input, unsigned int selected_line,
-                                        Switcher *sw )
+static ModeMode combi_mode_result ( Mode *sw, int mretv, char **input, unsigned int selected_line )
 {
-    CombiModePrivateData *pd = sw->private_data;
+    CombiModePrivateData *pd = mode_get_private_data ( sw );
     if ( *input[0] == '!' ) {
         int switcher = -1;
         for ( unsigned i = 0; switcher == -1 && i < pd->num_switchers; i++ ) {
-            if ( ( *input )[1] == pd->switchers[i]->name[0] ) {
+            if ( ( *input )[1] == mode_get_name ( pd->switchers[i] )[0] ) {
                 switcher = i;
             }
         }
@@ -171,9 +161,8 @@ static SwitcherMode combi_mode_result ( int mretv, char **input, unsigned int se
             // skip whitespace
             if ( n != NULL ) {
                 n++;
-                return pd->switchers[switcher]->result ( mretv, &n,
-                                                         selected_line - pd->starts[switcher],
-                                                         pd->switchers[switcher] );
+                return mode_result ( pd->switchers[switcher], mretv, &n,
+                                     selected_line - pd->starts[switcher] );
             }
             return MODE_EXIT;
         }
@@ -182,63 +171,94 @@ static SwitcherMode combi_mode_result ( int mretv, char **input, unsigned int se
     for ( unsigned i = 0; i < pd->num_switchers; i++ ) {
         if ( selected_line >= pd->starts[i] &&
              selected_line < ( pd->starts[i] + pd->lengths[i] ) ) {
-            return pd->switchers[i]->result ( mretv, input, selected_line - pd->starts[i],
-                                              pd->switchers[i] );
+            return mode_result ( pd->switchers[i], mretv, input, selected_line - pd->starts[i] );
         }
     }
     return MODE_EXIT;
 }
-static int combi_mode_match ( char **tokens, const char *input, int not_ascii,
-                              int case_sensitive, unsigned int index, Switcher *sw )
+static int combi_mode_match ( const Mode *sw, GRegex **tokens, unsigned int index )
 {
-    CombiModePrivateData *pd = sw->private_data;
-
+    CombiModePrivateData *pd = mode_get_private_data ( sw );
     for ( unsigned i = 0; i < pd->num_switchers; i++ ) {
+        if ( pd->current != NULL && pd->switchers[i] != pd->current ) {
+            continue;
+        }
         if ( index >= pd->starts[i] && index < ( pd->starts[i] + pd->lengths[i] ) ) {
-            if ( tokens && input[0] && tokens[0][0] == '!' ) {
-                if ( tokens[0][1] == pd->switchers[i]->name[0] ) {
-                    return pd->switchers[i]->token_match ( &tokens[1], input, not_ascii, case_sensitive,
-                                                           index - pd->starts[i], pd->switchers[i] );
-                }
-                return 0;
-            }
-            else {
-                return pd->switchers[i]->token_match ( tokens, input, not_ascii, case_sensitive,
-                                                       index - pd->starts[i], pd->switchers[i] );
-            }
+            return mode_token_match ( pd->switchers[i], tokens, index - pd->starts[i] );
         }
     }
-    abort ();
     return 0;
 }
-static const char * combi_mgrv ( unsigned int selected_line, void *sw, int *state )
+static char * combi_mgrv ( const Mode *sw, unsigned int selected_line, int *state, int get_entry )
 {
-    CombiModePrivateData *pd = ( (Switcher *) sw )->private_data;
+    CombiModePrivateData *pd = mode_get_private_data ( sw );
+    if ( !get_entry ) {
+        for ( unsigned i = 0; i < pd->num_switchers; i++ ) {
+            if ( selected_line >= pd->starts[i] && selected_line < ( pd->starts[i] + pd->lengths[i] ) ) {
+                mode_get_display_value ( pd->switchers[i], selected_line - pd->starts[i], state, FALSE );
+                return NULL;
+            }
+        }
+        return NULL;
+    }
     for ( unsigned i = 0; i < pd->num_switchers; i++ ) {
         if ( selected_line >= pd->starts[i] && selected_line < ( pd->starts[i] + pd->lengths[i] ) ) {
-            g_free ( pd->cache );
-            pd->cache = g_strdup_printf ( "(%s) %s", pd->switchers[i]->name,
-                                          pd->switchers[i]->mgrv ( selected_line - pd->starts[i],
-                                                                   (void *) pd->switchers[i], state ) );
-            return pd->cache;
+            char * str  = mode_get_display_value ( pd->switchers[i], selected_line - pd->starts[i], state, TRUE );
+            char * retv = g_strdup_printf ( "%s %s", mode_get_display_name ( pd->switchers[i] ), str );
+            g_free ( str );
+            return retv;
         }
     }
 
     return NULL;
 }
-
-Switcher combi_mode =
+static char * combi_get_completion ( const Mode *sw, unsigned int index )
 {
-    .name         = "combi",
-    .keycfg       = NULL,
-    .keystr       = NULL,
-    .modmask      = AnyModifier,
-    .init         = combi_mode_init,
-    .get_data     = combi_mode_get_data,
-    .result       = combi_mode_result,
-    .destroy      = combi_mode_destroy,
-    .token_match  = combi_mode_match,
-    .mgrv         = combi_mgrv,
-    .private_data = NULL,
-    .free         = NULL
+    CombiModePrivateData *pd = mode_get_private_data ( sw );
+    for ( unsigned i = 0; i < pd->num_switchers; i++ ) {
+        if ( index >= pd->starts[i] && index < ( pd->starts[i] + pd->lengths[i] ) ) {
+            char *comp  = mode_get_completion ( pd->switchers[i], index - pd->starts[i] );
+            char *mcomp = g_strdup_printf ( "!%c %s", mode_get_name ( pd->switchers[i] )[0], comp );
+            g_free ( comp );
+            return mcomp;
+        }
+    }
+    // Should never get here.
+    g_error ( "Failure, could not resolve sub-switcher." );
+    return NULL;
+}
+
+static char * combi_preprocess_input ( Mode *sw, const char *input )
+{
+    CombiModePrivateData *pd = mode_get_private_data ( sw );
+    pd->current = NULL;
+    if ( input != NULL && input[0] == '!' && strlen ( input ) > 1 ) {
+        for ( unsigned i = 0; i < pd->num_switchers; i++ ) {
+            if ( input[1] == mode_get_name ( pd->switchers[i] )[0] ) {
+                pd->current = pd->switchers[i];
+                if ( input[2] == '\0' ) {
+                    return NULL;
+                }
+                return g_strdup ( &input[2] );
+            }
+        }
+    }
+    return g_strdup ( input );
+}
+
+#include "mode-private.h"
+Mode combi_mode =
+{
+    .name               = "combi",
+    .cfg_name_key       = "display-combi",
+    ._init              = combi_mode_init,
+    ._get_num_entries   = combi_mode_get_num_entries,
+    ._result            = combi_mode_result,
+    ._destroy           = combi_mode_destroy,
+    ._token_match       = combi_mode_match,
+    ._get_completion    = combi_get_completion,
+    ._get_display_value = combi_mgrv,
+    ._preprocess_input  = combi_preprocess_input,
+    .private_data       = NULL,
+    .free               = NULL
 };
