@@ -1,8 +1,8 @@
-/**
+/*
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2015 Qball Cow <qball@gmpclient.org>
+ * Copyright 2013-2016 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -24,10 +24,14 @@
  * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  */
+
+/**
+ * \ingroup RUNMode
+ * @{
+ */
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <X11/X.h>
 
 #include <unistd.h>
 #include <limits.h>
@@ -39,12 +43,36 @@
 #include <errno.h>
 
 #include "rofi.h"
+#include "settings.h"
 #include "helper.h"
 #include "history.h"
 #include "dialogs/run.h"
 
-#define RUN_CACHE_FILE    "rofi-2.runcache"
+#include "mode-private.h"
+/**
+ * Name of the history file where previously choosen commands are stored.
+ */
+#define RUN_CACHE_FILE    "rofi-3.runcache"
 
+/**
+ * The internal data structure holding the private data of the Run Mode.
+ */
+typedef struct
+{
+    /** list of available commands. */
+    char         **cmd_list;
+    /** Length of the #cmd_list. */
+    unsigned int cmd_list_length;
+} RunModePrivateData;
+
+/**
+ * @param cmd The cmd to execute
+ * @param run_in_term Indicate if command should be run in a terminal
+ *
+ * Execute command.
+ *
+ * @returns FALSE On failure, TRUE on success
+ */
 static inline int execsh ( const char *cmd, int run_in_term )
 {
     int  retv   = TRUE;
@@ -60,7 +88,7 @@ static inline int execsh ( const char *cmd, int run_in_term )
     g_spawn_async ( NULL, args, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error );
     if ( error != NULL ) {
         char *msg = g_strdup_printf ( "Failed to execute: '%s'\nError: '%s'", cmd, error->message );
-        error_dialog ( msg, FALSE  );
+        rofi_view_error_dialog ( msg, FALSE  );
         g_free ( msg );
         // print error.
         g_error_free ( error );
@@ -72,39 +100,64 @@ static inline int execsh ( const char *cmd, int run_in_term )
     return retv;
 }
 
-// execute sub-process
+/**
+ * @param cmd The cmd to execute
+ * @param run_in_term Indicate if command should be run in a terminal
+ *
+ * Execute command and add to history.
+ */
 static void exec_cmd ( const char *cmd, int run_in_term )
 {
+    GError *error = NULL;
     if ( !cmd || !cmd[0] ) {
         return;
     }
+    gsize lf_cmd_size = 0;
+    gchar *lf_cmd     = g_locale_from_utf8 ( cmd, -1, NULL, &lf_cmd_size, &error );
+    if ( error != NULL ) {
+        fprintf ( stderr, "Failed to convert command to locale encoding: %s\n", error->message );
+        g_error_free ( error );
+        return;
+    }
 
-    if (  execsh ( cmd, run_in_term ) ) {
+    if (  execsh ( lf_cmd, run_in_term ) ) {
         /**
          * This happens in non-critical time (After launching app)
          * It is allowed to be a bit slower.
          */
-        char *path = g_strdup_printf ( "%s/%s", cache_dir, RUN_CACHE_FILE );
+        char *path = g_build_filename ( cache_dir, RUN_CACHE_FILE, NULL );
 
         history_set ( path, cmd );
 
         g_free ( path );
     }
+    g_free ( lf_cmd );
 }
-// execute sub-process
+
+/**
+ * @param cmd The command to remove from history
+ *
+ * Remove command from history.
+ */
 static void delete_entry ( const char *cmd )
 {
-    /**
-     * This happens in non-critical time (After launching app)
-     * It is allowed to be a bit slower.
-     */
-    char *path = g_strdup_printf ( "%s/%s", cache_dir, RUN_CACHE_FILE );
+    char *path = g_build_filename ( cache_dir, RUN_CACHE_FILE, NULL );
 
     history_remove ( path, cmd );
 
     g_free ( path );
 }
-static int sort_func ( const void *a, const void *b, void *data __attribute__( ( unused ) ) )
+
+/**
+ * @param a The First key to compare
+ * @param b The second key to compare
+ * @param data Unused.
+ *
+ * Function used for sorting.
+ *
+ * @returns returns less then, equal to and greater than zero is a is less than, is a match or greater than b.
+ */
+static int sort_func ( const void *a, const void *b, G_GNUC_UNUSED void *data )
 {
     const char *astr = *( const char * const * ) a;
     const char *bstr = *( const char * const * ) b;
@@ -118,7 +171,7 @@ static int sort_func ( const void *a, const void *b, void *data __attribute__( (
     else if ( bstr == NULL ) {
         return -1;
     }
-    return strcasecmp ( astr, bstr );
+    return g_ascii_strcasecmp ( astr, bstr );
 }
 
 /**
@@ -130,8 +183,10 @@ static char ** get_apps_external ( char **retv, unsigned int *length, unsigned i
     if ( fd >= 0 ) {
         FILE *inp = fdopen ( fd, "r" );
         if ( inp ) {
-            char buffer[1024];
-            while ( fgets ( buffer, 1024, inp ) != NULL ) {
+            char   *buffer       = NULL;
+            size_t buffer_length = 0;
+
+            while ( getline ( &buffer, &buffer_length, inp ) > 0 ) {
                 int found = 0;
                 // Filter out line-end.
                 if ( buffer[strlen ( buffer ) - 1] == '\n' ) {
@@ -156,6 +211,9 @@ static char ** get_apps_external ( char **retv, unsigned int *length, unsigned i
 
                 ( *length )++;
             }
+            if ( buffer != NULL ) {
+                free ( buffer );
+            }
             if ( fclose ( inp ) != 0 ) {
                 fprintf ( stderr, "Failed to close stdout off executor script: '%s'\n",
                           strerror ( errno ) );
@@ -171,6 +229,7 @@ static char ** get_apps_external ( char **retv, unsigned int *length, unsigned i
  */
 static char ** get_apps ( unsigned int *length )
 {
+    GError       *error        = NULL;
     char         **retv        = NULL;
     unsigned int num_favorites = 0;
     char         *path;
@@ -178,20 +237,39 @@ static char ** get_apps ( unsigned int *length )
     if ( g_getenv ( "PATH" ) == NULL ) {
         return NULL;
     }
-
-    path = g_strdup_printf ( "%s/%s", cache_dir, RUN_CACHE_FILE );
+    TICK_N ( "start" );
+    path = g_build_filename ( cache_dir, RUN_CACHE_FILE, NULL );
     retv = history_get_list ( path, length );
     g_free ( path );
     // Keep track of how many where loaded as favorite.
     num_favorites = ( *length );
 
-    path = g_strdup ( getenv ( "PATH" ) );
+    path = g_strdup ( g_getenv ( "PATH" ) );
 
-    for ( const char *dirname = strtok ( path, ":" ); dirname != NULL; dirname = strtok ( NULL, ":" ) ) {
+    gsize l        = 0;
+    gchar *homedir = g_locale_to_utf8 (  g_get_home_dir (), -1, NULL, &l, &error );
+    if ( error != NULL ) {
+        fprintf ( stderr, "Failed to convert homedir to UTF-8: %s\n", error->message );
+        g_clear_error ( &error );
+        g_free ( homedir );
+        return NULL;
+    }
+
+    const char *const sep = ":";
+    for ( const char *dirname = strtok ( path, sep ); dirname != NULL; dirname = strtok ( NULL, sep ) ) {
         DIR *dir = opendir ( dirname );
 
         if ( dir != NULL ) {
             struct dirent *dent;
+            gsize         dirn_len = 0;
+            gchar         *dirn    = g_locale_to_utf8 ( dirname, -1, NULL, &dirn_len, &error );
+            if ( error != NULL ) {
+                fprintf ( stderr, "Failed to convert directory name to UTF-8: %s\n", error->message );
+                g_clear_error ( &error );
+                continue;
+            }
+            gboolean is_homedir = g_str_has_prefix ( dirn, homedir );
+            g_free ( dirn );
 
             while ( ( dent = readdir ( dir ) ) != NULL ) {
                 if ( dent->d_type != DT_REG && dent->d_type != DT_LNK && dent->d_type != DT_UNKNOWN ) {
@@ -201,23 +279,39 @@ static char ** get_apps ( unsigned int *length )
                 if ( dent->d_name[0] == '.' ) {
                     continue;
                 }
+                if ( is_homedir ) {
+                    gchar    *fpath = g_build_filename ( dirname, dent->d_name, NULL );
+                    gboolean b      = g_file_test ( fpath, G_FILE_TEST_IS_EXECUTABLE );
+                    g_free ( fpath );
+                    if ( !b ) {
+                        continue;
+                    }
+                }
 
-                int found = 0;
-
+                gsize name_len;
+                gchar *name = g_filename_to_utf8 ( dent->d_name, -1, NULL, &name_len, &error );
+                if ( error != NULL ) {
+                    fprintf ( stderr, "Failed to convert filename to UTF-8: %s\n", error->message );
+                    g_clear_error ( &error );
+                    g_free ( name );
+                    continue;
+                }
                 // This is a nice little penalty, but doable? time will tell.
                 // given num_favorites is max 25.
+                int found = 0;
                 for ( unsigned int j = 0; found == 0 && j < num_favorites; j++ ) {
-                    if ( strcasecmp ( dent->d_name, retv[j] ) == 0 ) {
+                    if ( g_strcmp0 ( name, retv[j] ) == 0 ) {
                         found = 1;
                     }
                 }
 
                 if ( found == 1 ) {
+                    g_free ( name );
                     continue;
                 }
 
                 retv                  = g_realloc ( retv, ( ( *length ) + 2 ) * sizeof ( char* ) );
-                retv[( *length )]     = g_strdup ( dent->d_name );
+                retv[( *length )]     = name;
                 retv[( *length ) + 1] = NULL;
                 ( *length )++;
             }
@@ -225,6 +319,7 @@ static char ** get_apps ( unsigned int *length )
             closedir ( dir );
         }
     }
+    g_free ( homedir );
 
     // Get external apps.
     if ( config.run_list_command != NULL && config.run_list_command[0] != '\0' ) {
@@ -257,44 +352,42 @@ static char ** get_apps ( unsigned int *length )
     // Reduce array length;
     ( *length ) -= removed;
 
+    TICK_N ( "stop" );
     return retv;
 }
 
-typedef struct _RunModePrivateData
-{
-    unsigned int id;
-    char         **cmd_list;
-    unsigned int cmd_list_length;
-} RunModePrivateData;
-
-static void run_mode_init ( Switcher *sw )
+static int run_mode_init ( Mode *sw )
 {
     if ( sw->private_data == NULL ) {
         RunModePrivateData *pd = g_malloc0 ( sizeof ( *pd ) );
         sw->private_data = (void *) pd;
+        pd->cmd_list     = get_apps ( &( pd->cmd_list_length ) );
+    }
+
+    return TRUE;
+}
+static void run_mode_destroy ( Mode *sw )
+{
+    RunModePrivateData *rmpd = (RunModePrivateData *) sw->private_data;
+    if ( rmpd != NULL ) {
+        g_strfreev ( rmpd->cmd_list );
+        g_free ( rmpd );
+        sw->private_data = NULL;
     }
 }
 
-static char ** run_mode_get_data ( unsigned int *length, Switcher *sw )
+static unsigned int run_mode_get_num_entries ( const Mode *sw )
 {
-    RunModePrivateData *rmpd = (RunModePrivateData *) sw->private_data;
-    if ( rmpd->cmd_list == NULL ) {
-        rmpd->cmd_list_length = 0;
-        rmpd->cmd_list        = get_apps ( &( rmpd->cmd_list_length ) );
-    }
-    if ( length != NULL ) {
-        *length = rmpd->cmd_list_length;
-    }
-    return rmpd->cmd_list;
+    const RunModePrivateData *rmpd = (const RunModePrivateData *) sw->private_data;
+    return rmpd->cmd_list_length;
 }
 
-static SwitcherMode run_mode_result ( int mretv, char **input, unsigned int selected_line,
-                                      Switcher *sw )
+static ModeMode run_mode_result ( Mode *sw, int mretv, char **input, unsigned int selected_line )
 {
     RunModePrivateData *rmpd = (RunModePrivateData *) sw->private_data;
-    SwitcherMode       retv  = MODE_EXIT;
+    ModeMode           retv  = MODE_EXIT;
 
-    int                shift = ( ( mretv & MENU_SHIFT ) == MENU_SHIFT );
+    gboolean           run_in_term = ( ( mretv & MENU_CUSTOM_ACTION ) == MENU_CUSTOM_ACTION );
 
     if ( mretv & MENU_NEXT ) {
         retv = NEXT_DIALOG;
@@ -306,50 +399,47 @@ static SwitcherMode run_mode_result ( int mretv, char **input, unsigned int sele
         retv = ( mretv & MENU_LOWER_MASK );
     }
     else if ( ( mretv & MENU_OK ) && rmpd->cmd_list[selected_line] != NULL ) {
-        exec_cmd ( rmpd->cmd_list[selected_line], shift );
+        exec_cmd ( rmpd->cmd_list[selected_line], run_in_term );
     }
     else if ( ( mretv & MENU_CUSTOM_INPUT ) && *input != NULL && *input[0] != '\0' ) {
-        exec_cmd ( *input, shift );
+        exec_cmd ( *input, run_in_term );
     }
     else if ( ( mretv & MENU_ENTRY_DELETE ) && rmpd->cmd_list[selected_line] ) {
         delete_entry ( rmpd->cmd_list[selected_line] );
 
-        g_free ( rmpd->cmd_list );
         // Clear the list.
-        rmpd->cmd_list        = NULL;
-        rmpd->cmd_list_length = 0;
-        retv                  = RELOAD_DIALOG;
+        retv = RELOAD_DIALOG;
+        run_mode_destroy ( sw );
+        run_mode_init ( sw );
     }
     return retv;
 }
 
-static void run_mode_destroy ( Switcher *sw )
+static char *_get_display_value ( const Mode *sw, unsigned int selected_line, G_GNUC_UNUSED int *state, int get_entry )
 {
-    RunModePrivateData *rmpd = (RunModePrivateData *) sw->private_data;
-    if ( rmpd != NULL ) {
-        g_strfreev ( rmpd->cmd_list );
-        g_free ( rmpd );
-        sw->private_data = NULL;
-    }
+    const RunModePrivateData *rmpd = (const RunModePrivateData *) sw->private_data;
+    return get_entry ? g_strdup ( rmpd->cmd_list[selected_line] ) : NULL;
+}
+static int run_token_match ( const Mode *sw, GRegex **tokens, unsigned int index )
+{
+    const RunModePrivateData *rmpd = (const RunModePrivateData *) sw->private_data;
+    return token_match ( tokens, rmpd->cmd_list[index] );
 }
 
-static const char *mgrv ( unsigned int selected_line, void *sw, G_GNUC_UNUSED int *state )
+#include "mode-private.h"
+Mode run_mode =
 {
-    return run_mode_get_data ( NULL, sw )[selected_line];
-}
-
-Switcher run_mode =
-{
-    .name         = "run",
-    .keycfg       = NULL,
-    .keystr       = NULL,
-    .modmask      = AnyModifier,
-    .init         = run_mode_init,
-    .get_data     = run_mode_get_data,
-    .result       = run_mode_result,
-    .destroy      = run_mode_destroy,
-    .token_match  = token_match,
-    .mgrv         = mgrv,
-    .private_data = NULL,
-    .free         = NULL
+    .name               = "run",
+    .cfg_name_key       = "display-run",
+    ._init              = run_mode_init,
+    ._get_num_entries   = run_mode_get_num_entries,
+    ._result            = run_mode_result,
+    ._destroy           = run_mode_destroy,
+    ._token_match       = run_token_match,
+    ._get_display_value = _get_display_value,
+    ._get_completion    = NULL,
+    ._preprocess_input  = NULL,
+    .private_data       = NULL,
+    .free               = NULL
 };
+/*@}*/
