@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2015 Qball Cow <qball@gmpclient.org>
+ * Copyright 2013-2016 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -38,6 +38,7 @@
 #include "dialogs/script.h"
 #include "helper.h"
 
+#include "mode-private.h"
 static char **get_script_output ( const char *command, unsigned int *length )
 {
     char **retv = NULL;
@@ -47,8 +48,9 @@ static char **get_script_output ( const char *command, unsigned int *length )
     if ( fd >= 0 ) {
         FILE *inp = fdopen ( fd, "r" );
         if ( inp ) {
-            char buffer[1024];
-            while ( fgets ( buffer, 1024, inp ) != NULL ) {
+            char   *buffer       = NULL;
+            size_t buffer_length = 0;
+            while ( getline ( &buffer, &buffer_length, inp ) > 0 ) {
                 retv                  = g_realloc ( retv, ( ( *length ) + 2 ) * sizeof ( char* ) );
                 retv[( *length )]     = g_strdup ( buffer );
                 retv[( *length ) + 1] = NULL;
@@ -60,6 +62,9 @@ static char **get_script_output ( const char *command, unsigned int *length )
 
                 ( *length )++;
             }
+            if ( buffer ) {
+                free ( buffer );
+            }
             if ( fclose ( inp ) != 0 ) {
                 fprintf ( stderr, "Failed to close stdout off executor script: '%s'\n",
                           strerror ( errno ) );
@@ -69,7 +74,7 @@ static char **get_script_output ( const char *command, unsigned int *length )
     return retv;
 }
 
-static char **execute_executor ( Switcher *sw, const char *result, unsigned int *length )
+static char **execute_executor ( Mode *sw, const char *result, unsigned int *length )
 {
     char **retv = NULL;
 
@@ -81,7 +86,7 @@ static char **execute_executor ( Switcher *sw, const char *result, unsigned int
     return retv;
 }
 
-static void script_switcher_free ( Switcher *sw )
+static void script_switcher_free ( Mode *sw )
 {
     if ( sw == NULL ) {
         return;
@@ -90,36 +95,32 @@ static void script_switcher_free ( Switcher *sw )
     g_free ( sw );
 }
 
-typedef struct _ScriptModePrivateData
+typedef struct
 {
     unsigned int id;
     char         **cmd_list;
     unsigned int cmd_list_length;
 } ScriptModePrivateData;
 
-static void script_mode_init ( Switcher *sw )
+static int script_mode_init ( Mode *sw )
 {
     if ( sw->private_data == NULL ) {
         ScriptModePrivateData *pd = g_malloc0 ( sizeof ( *pd ) );
         sw->private_data = (void *) pd;
+        pd->cmd_list     = get_script_output ( (const char *) sw->ed, &( pd->cmd_list_length ) );
     }
+    return TRUE;
 }
-static char ** script_mode_get_data ( unsigned int *length, Switcher *sw )
+static unsigned int script_mode_get_num_entries ( const Mode *sw )
 {
-    ScriptModePrivateData *rmpd = (ScriptModePrivateData *) sw->private_data;
-    if ( rmpd->cmd_list == NULL ) {
-        rmpd->cmd_list_length = 0;
-        rmpd->cmd_list        = get_script_output ( (const char *) sw->ed, &( rmpd->cmd_list_length ) );
-    }
-    *length = rmpd->cmd_list_length;
-    return rmpd->cmd_list;
+    const ScriptModePrivateData *rmpd = (const ScriptModePrivateData *) sw->private_data;
+    return rmpd->cmd_list_length;
 }
 
-static SwitcherMode script_mode_result ( int mretv, char **input, unsigned int selected_line,
-                                         Switcher *sw )
+static ModeMode script_mode_result ( Mode *sw, int mretv, char **input, unsigned int selected_line )
 {
     ScriptModePrivateData *rmpd      = (ScriptModePrivateData *) sw->private_data;
-    SwitcherMode          retv       = MODE_EXIT;
+    ModeMode              retv       = MODE_EXIT;
     char                  **new_list = NULL;
     unsigned int          new_length = 0;
 
@@ -152,7 +153,7 @@ static SwitcherMode script_mode_result ( int mretv, char **input, unsigned int s
     return retv;
 }
 
-static void script_mode_destroy ( Switcher *sw )
+static void script_mode_destroy ( Mode *sw )
 {
     ScriptModePrivateData *rmpd = (ScriptModePrivateData *) sw->private_data;
     if ( rmpd != NULL ) {
@@ -161,38 +162,46 @@ static void script_mode_destroy ( Switcher *sw )
         sw->private_data = NULL;
     }
 }
-static const char *mgrv ( unsigned int selected_line, void *sw, G_GNUC_UNUSED int *state )
+static char *_get_display_value ( const Mode *sw, unsigned int selected_line, G_GNUC_UNUSED int *state, int get_entry )
 {
-    ScriptModePrivateData *rmpd = ( (Switcher *) sw )->private_data;
-    return rmpd->cmd_list[selected_line];
+    ScriptModePrivateData *rmpd = sw->private_data;
+    return get_entry ? g_strdup ( rmpd->cmd_list[selected_line] ) : NULL;
 }
 
-Switcher *script_switcher_parse_setup ( const char *str )
+static int script_token_match ( const Mode *sw, GRegex **tokens, unsigned int index )
 {
-    Switcher     *sw    = g_malloc0 ( sizeof ( *sw ) );
-    char         *endp  = NULL;
-    char         *parse = g_strdup ( str );
-    unsigned int index  = 0;
-    for ( char *token = strtok_r ( parse, ":", &endp ); token != NULL; token = strtok_r ( NULL, ":", &endp ) ) {
+    ScriptModePrivateData *rmpd = sw->private_data;
+    return token_match ( tokens, rmpd->cmd_list[index] );
+}
+
+#include "mode-private.h"
+Mode *script_switcher_parse_setup ( const char *str )
+{
+    Mode              *sw    = g_malloc0 ( sizeof ( *sw ) );
+    char              *endp  = NULL;
+    char              *parse = g_strdup ( str );
+    unsigned int      index  = 0;
+    const char *const sep    = ":";
+    for ( char *token = strtok_r ( parse, sep, &endp ); token != NULL; token = strtok_r ( NULL, sep, &endp ) ) {
         if ( index == 0 ) {
             g_strlcpy ( sw->name, token, 32 );
         }
         else if ( index == 1 ) {
-            sw->ed = (void *) g_strdup ( token );
+            sw->ed = (void *) rofi_expand_path ( token );
         }
         index++;
     }
     g_free ( parse );
     if ( index == 2 ) {
-        sw->free        = script_switcher_free;
-        sw->keysym      = None;
-        sw->modmask     = AnyModifier;
-        sw->init        = script_mode_init;
-        sw->get_data    = script_mode_get_data;
-        sw->result      = script_mode_result;
-        sw->destroy     = script_mode_destroy;
-        sw->token_match = token_match;
-        sw->mgrv        = mgrv;
+        sw->free               = script_switcher_free;
+        sw->_init              = script_mode_init;
+        sw->_get_num_entries   = script_mode_get_num_entries;
+        sw->_result            = script_mode_result;
+        sw->_destroy           = script_mode_destroy;
+        sw->_token_match       = script_token_match;
+        sw->_get_completion    = NULL,
+        sw->_preprocess_input  = NULL,
+        sw->_get_display_value = _get_display_value;
 
         return sw;
     }
