@@ -1,8 +1,8 @@
-/**
+/*
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2015 Qball Cow <qball@gmpclient.org>
+ * Copyright 2013-2016 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -25,10 +25,13 @@
  *
  */
 
+/**
+ * \ingroup SSHMode
+ * @{
+ */
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <X11/X.h>
 
 #include <unistd.h>
 #include <signal.h>
@@ -41,15 +44,28 @@
 #include <helper.h>
 
 #include "rofi.h"
+#include "settings.h"
 #include "history.h"
 #include "dialogs/ssh.h"
 
-#define SSH_CACHE_FILE    "rofi-2.sshcache"
+/**
+ * Name of the history file where previously choosen hosts are stored.
+ */
+#define SSH_CACHE_FILE     "rofi-2.sshcache"
 
-// Used in get_ssh() when splitting lines from the user's
-// SSH config file into tokens.
+/**
+ * Used in get_ssh() when splitting lines from the user's
+ * SSH config file into tokens.
+ */
 #define SSH_TOKEN_DELIM    "= \t\r\n"
 
+/**
+ * @param host The host to connect too
+ *
+ * SSH into the selected host.
+ *
+ * @returns FALSE On failure, TRUE on success
+ */
 static inline int execshssh ( const char *host )
 {
     char **args = NULL;
@@ -61,68 +77,74 @@ static inline int execshssh ( const char *host )
 
     if ( error != NULL ) {
         char *msg = g_strdup_printf ( "Failed to execute: 'ssh %s'\nError: '%s'", host, error->message );
-        error_dialog ( msg, FALSE  );
+        rofi_view_error_dialog ( msg, FALSE  );
         g_free ( msg );
         // print error.
         g_error_free ( error );
+
+        g_strfreev ( args );
+        return FALSE;
     }
     // Free the args list.
     g_strfreev ( args );
 
-    return 0;
+    return TRUE;
 }
-// execute sub-process
-static pid_t exec_ssh ( const char *cmd )
+
+/**
+ * @param host The host to connect too
+ *
+ * SSH into the selected host, if successful update history.
+ */
+static void exec_ssh ( const char *host )
 {
-    if ( !cmd || !cmd[0] ) {
-        return -1;
-    }
-
-    execshssh ( cmd );
-
-    /**
-     * This happens in non-critical time (After launching app)
-     * It is allowed to be a bit slower.
-     */
-    char *path = g_strdup_printf ( "%s/%s", cache_dir, SSH_CACHE_FILE );
-    history_set ( path, cmd );
-    g_free ( path );
-
-    return 0;
-}
-static void delete_ssh ( const char *cmd )
-{
-    if ( !cmd || !cmd[0] ) {
+    if ( !host || !host[0] ) {
         return;
     }
-    char *path = NULL;
-    path = g_strdup_printf ( "%s/%s", cache_dir, SSH_CACHE_FILE );
-    history_remove ( path, cmd );
+
+    if ( !execshssh ( host ) ) {
+        return;
+    }
+
+    //  This happens in non-critical time (After launching app)
+    //  It is allowed to be a bit slower.
+    char *path = g_build_filename ( cache_dir, SSH_CACHE_FILE, NULL );
+    history_set ( path, host );
     g_free ( path );
 }
-static int ssh_sort_func ( const void *a, const void *b, void *data __attribute__( ( unused ) ) )
+
+/**
+ * @param host The host to remove from history
+ *
+ * Remove host from history.
+ */
+static void delete_ssh ( const char *host )
 {
-    const char *astr = *( const char * const * ) a;
-    const char *bstr = *( const char * const * ) b;
-    return g_utf8_collate ( astr, bstr );
+    if ( !host || !host[0] ) {
+        return;
+    }
+    char *path = g_build_filename ( cache_dir, SSH_CACHE_FILE, NULL );
+    history_remove ( path, host );
+    g_free ( path );
 }
 
 /**
  * @param retv list of hosts
- * @pwaram length pointer to length of list
+ * @param length pointer to length of list [in][out]
  *
  * Read 'known_hosts' file when entries are not hashsed.
  *
- * @returns list of hosts.
+ * @returns updated list of hosts.
  */
 static char **read_known_hosts_file ( char ** retv, unsigned int *length )
 {
     char *path = g_build_filename ( g_getenv ( "HOME" ), ".ssh", "known_hosts", NULL );
     FILE *fd   = fopen ( path, "r" );
     if ( fd != NULL ) {
-        char buffer[1024];
+        char   *buffer       = NULL;
+        size_t buffer_length = 0;
         // Reading one line per time.
-        while ( fgets ( buffer, sizeof ( buffer ), fd ) ) {
+        while ( getline ( &buffer, &buffer_length, fd ) > 0 ) {
             char *sep = strstr ( buffer, "," );
 
             if ( sep != NULL ) {
@@ -146,6 +168,9 @@ static char **read_known_hosts_file ( char ** retv, unsigned int *length )
                 }
             }
         }
+        if ( buffer != NULL ) {
+            free ( buffer );
+        }
         if ( fclose ( fd ) != 0 ) {
             fprintf ( stderr, "Failed to close hosts file: '%s'\n", strerror ( errno ) );
         }
@@ -154,17 +179,24 @@ static char **read_known_hosts_file ( char ** retv, unsigned int *length )
     g_free ( path );
     return retv;
 }
+
 /**
- * Read /etc/hosts
+ * @param retv The list of hosts to update.
+ * @param length The length of the list retv [in][out]
+ *
+ * Read `/etc/hosts` and appends them to the list retv
+ *
+ * @returns an updated list with the added hosts.
  */
 static char **read_hosts_file ( char ** retv, unsigned int *length )
 {
     // Read the hosts file.
     FILE *fd = fopen ( "/etc/hosts", "r" );
     if ( fd != NULL ) {
-        char buffer[1024];
+        char   *buffer       = NULL;
+        size_t buffer_length = 0;
         // Reading one line per time.
-        while ( fgets ( buffer, sizeof ( buffer ), fd ) ) {
+        while ( getline ( &buffer, &buffer_length, fd ) > 0 ) {
             // Evaluate one line.
             unsigned int index  = 0, ti = 0;
             char         *token = buffer;
@@ -211,6 +243,9 @@ static char **read_hosts_file ( char ** retv, unsigned int *length )
                 index++;
             } while ( buffer[index] != '\0' && buffer[index] != '#' );
         }
+        if ( buffer != NULL ) {
+            free ( buffer );
+        }
         if ( fclose ( fd ) != 0 ) {
             fprintf ( stderr, "Failed to close hosts file: '%s'\n", strerror ( errno ) );
         }
@@ -219,6 +254,13 @@ static char **read_hosts_file ( char ** retv, unsigned int *length )
     return retv;
 }
 
+/**
+ * @param length The number of found ssh hosts [out]
+ *
+ * Gets the list available SSH hosts.
+ *
+ * @return an array of strings containing all the hosts.
+ */
 static char ** get_ssh (  unsigned int *length )
 {
     char         **retv        = NULL;
@@ -229,7 +271,7 @@ static char ** get_ssh (  unsigned int *length )
         return NULL;
     }
 
-    path = g_strdup_printf ( "%s/%s", cache_dir, SSH_CACHE_FILE );
+    path = g_build_filename ( cache_dir, SSH_CACHE_FILE, NULL );
     retv = history_get_list ( path, length );
     g_free ( path );
     num_favorites = ( *length );
@@ -243,12 +285,13 @@ static char ** get_ssh (  unsigned int *length )
 
     FILE       *fd = NULL;
     const char *hd = getenv ( "HOME" );
-    path = g_strdup_printf ( "%s/%s", hd, ".ssh/config" );
+    path = g_build_filename ( hd, ".ssh", "config", NULL );
     fd   = fopen ( path, "r" );
 
     if ( fd != NULL ) {
-        char buffer[1024];
-        while ( fgets ( buffer, sizeof ( buffer ), fd ) ) {
+        char   *buffer       = NULL;
+        size_t buffer_length = 0;
+        while ( getline ( &buffer, &buffer_length, fd ) > 0 ) {
             // Each line is either empty, a comment line starting with a '#'
             // character or of the form "keyword [=] arguments", where there may
             // be multiple (possibly quoted) arguments separated by whitespace.
@@ -269,7 +312,8 @@ static char ** get_ssh (  unsigned int *length )
             // (how many host names contain spaces?).
             while ( ( token = strtok ( NULL, SSH_TOKEN_DELIM ) ) ) {
                 // We do not want to show wildcard entries, as you cannot ssh to them.
-                if ( *token == '!' || strpbrk ( token, "*?" ) ) {
+                const char *const sep = "*?";
+                if ( *token == '!' || strpbrk ( token, sep ) ) {
                     continue;
                 }
 
@@ -300,53 +344,74 @@ static char ** get_ssh (  unsigned int *length )
                 ( *length )++;
             }
         }
+        if ( buffer != NULL ) {
+            free ( buffer );
+        }
 
         if ( fclose ( fd ) != 0 ) {
             fprintf ( stderr, "Failed to close ssh configuration file: '%s'\n", strerror ( errno ) );
         }
     }
 
-    // TODO: check this is still fast enough. (takes 1ms on laptop.)
-    if ( ( *length ) > num_favorites ) {
-        g_qsort_with_data ( &retv[num_favorites], ( *length ) - num_favorites, sizeof ( char* ), ssh_sort_func, NULL );
-    }
     g_free ( path );
 
     return retv;
 }
 
-typedef struct _SSHModePrivateData
+/**
+ * The internal data structure holding the private data of the SSH Mode.
+ */
+typedef struct
 {
-    unsigned int id;
-    char         **cmd_list;
-    unsigned int cmd_list_length;
+    /** List if available ssh hosts.*/
+    char         **hosts_list;
+    /** Length of the #hosts_list.*/
+    unsigned int hosts_list_length;
 } SSHModePrivateData;
 
-static void ssh_mode_init ( Switcher *sw )
+/**
+ * @param sw Object handle to the SSH Mode object
+ *
+ * Initializes the SSH Mode private data object and
+ * loads the relevant ssh information.
+ */
+static int ssh_mode_init ( Mode *sw )
 {
-    if ( sw->private_data == NULL ) {
+    if ( mode_get_private_data ( sw ) == NULL ) {
         SSHModePrivateData *pd = g_malloc0 ( sizeof ( *pd ) );
-        sw->private_data = (void *) pd;
+        mode_set_private_data ( sw, (void *) pd );
+        pd->hosts_list = get_ssh ( &( pd->hosts_list_length ) );
     }
+    return TRUE;
 }
 
-static char ** ssh_mode_get_data ( unsigned int *length, Switcher *sw )
+/**
+ * @param sw Object handle to the SSH Mode object
+ *
+ * Get the number of SSH entries.
+ *
+ * @returns the number of ssh entries.
+ */
+static unsigned int ssh_mode_get_num_entries ( const Mode *sw )
 {
-    SSHModePrivateData *rmpd = (SSHModePrivateData *) sw->private_data;
-    if ( rmpd->cmd_list == NULL ) {
-        rmpd->cmd_list_length = 0;
-        rmpd->cmd_list        = get_ssh ( &( rmpd->cmd_list_length ) );
-    }
-    if ( length != NULL ) {
-        *length = rmpd->cmd_list_length;
-    }
-    return rmpd->cmd_list;
+    const SSHModePrivateData *rmpd = (const SSHModePrivateData *) mode_get_private_data ( sw );
+    return rmpd->hosts_list_length;
 }
-static SwitcherMode ssh_mode_result ( int mretv, char **input, unsigned int selected_line,
-                                      Switcher *sw )
+
+/**
+ * @param sw Object handle to the SSH Mode object
+ * @param mretv The menu return value.
+ * @param input Pointer to the user input string.
+ * @param selected_line the line selected by the user.
+ *
+ * Acts on the user interaction.
+ *
+ * @returns the next #ModeMode.
+ */
+static ModeMode ssh_mode_result ( Mode *sw, int mretv, char **input, unsigned int selected_line )
 {
-    SwitcherMode       retv  = MODE_EXIT;
-    SSHModePrivateData *rmpd = (SSHModePrivateData *) sw->private_data;
+    ModeMode           retv  = MODE_EXIT;
+    SSHModePrivateData *rmpd = (SSHModePrivateData *) mode_get_private_data ( sw );
     if ( mretv & MENU_NEXT ) {
         retv = NEXT_DIALOG;
     }
@@ -356,50 +421,83 @@ static SwitcherMode ssh_mode_result ( int mretv, char **input, unsigned int sele
     else if ( mretv & MENU_QUICK_SWITCH ) {
         retv = ( mretv & MENU_LOWER_MASK );
     }
-    else if ( ( mretv & MENU_OK ) && rmpd->cmd_list[selected_line] != NULL ) {
-        exec_ssh ( rmpd->cmd_list[selected_line] );
+    else if ( ( mretv & MENU_OK ) && rmpd->hosts_list[selected_line] != NULL ) {
+        exec_ssh ( rmpd->hosts_list[selected_line] );
     }
     else if ( ( mretv & MENU_CUSTOM_INPUT ) && *input != NULL && *input[0] != '\0' ) {
         exec_ssh ( *input );
     }
-    else if ( ( mretv & MENU_ENTRY_DELETE ) && rmpd->cmd_list[selected_line] ) {
-        delete_ssh ( rmpd->cmd_list[selected_line] );
-        g_strfreev ( rmpd->cmd_list );
-        rmpd->cmd_list_length = 0;
-        rmpd->cmd_list        = NULL;
+    else if ( ( mretv & MENU_ENTRY_DELETE ) && rmpd->hosts_list[selected_line] ) {
+        delete_ssh ( rmpd->hosts_list[selected_line] );
+        g_strfreev ( rmpd->hosts_list );
+        rmpd->hosts_list_length = 0;
+        rmpd->hosts_list        = NULL;
         // Stay
         retv = RELOAD_DIALOG;
     }
     return retv;
 }
 
-static void ssh_mode_destroy ( Switcher *sw )
+/**
+ * @param sw Object handle to the SSH Mode object
+ *
+ * Cleanup the SSH Mode. Free all allocated memory and NULL the private data pointer.
+ */
+static void ssh_mode_destroy ( Mode *sw )
 {
-    SSHModePrivateData *rmpd = (SSHModePrivateData *) sw->private_data;
+    SSHModePrivateData *rmpd = (SSHModePrivateData *) mode_get_private_data ( sw );
     if ( rmpd != NULL ) {
-        g_strfreev ( rmpd->cmd_list );
+        g_strfreev ( rmpd->hosts_list );
         g_free ( rmpd );
-        sw->private_data = NULL;
+        mode_set_private_data ( sw, NULL );
     }
 }
 
-static const char *mgrv ( unsigned int selected_line, void *sw, G_GNUC_UNUSED int *state )
+/**
+ * @param sw Object handle to the SSH Mode object
+ * @param selected_line The line to view
+ * @param state The state of the entry [out]
+ * @param get_entry
+ *
+ * Gets the string as it should be displayed and the display state.
+ * If get_entry is FALSE only the state is set.
+ *
+ * @return the string as it should be displayed and the display state.
+ */
+static char *_get_display_value ( const Mode *sw, unsigned int selected_line, G_GNUC_UNUSED int *state, int get_entry )
 {
-    return ssh_mode_get_data ( NULL, sw )[selected_line];
+    SSHModePrivateData *rmpd = (SSHModePrivateData *) mode_get_private_data ( sw );
+    return get_entry ? g_strdup ( rmpd->hosts_list[selected_line] ) : NULL;
 }
 
-Switcher ssh_mode =
+/**
+ * @param sw Object handle to the SSH Mode object
+ * @param tokens The set of tokens to match against
+ * @param index The index of the entry to match
+ *
+ * Match entry against the set of tokens.
+ *
+ * @returns TRUE if matches
+ */
+static int ssh_token_match ( const Mode *sw, GRegex **tokens, unsigned int index )
 {
-    .name         = "ssh",
-    .keycfg       = NULL,
-    .keystr       = NULL,
-    .modmask      = AnyModifier,
-    .init         = ssh_mode_init,
-    .get_data     = ssh_mode_get_data,
-    .result       = ssh_mode_result,
-    .destroy      = ssh_mode_destroy,
-    .token_match  = token_match,
-    .mgrv         = mgrv,
-    .private_data = NULL,
-    .free         = NULL
+    SSHModePrivateData *rmpd = (SSHModePrivateData *) mode_get_private_data ( sw );
+    return token_match ( tokens, rmpd->hosts_list[index] );
+}
+#include "mode-private.h"
+Mode ssh_mode =
+{
+    .name               = "ssh",
+    .cfg_name_key       = "display-ssh",
+    ._init              = ssh_mode_init,
+    ._get_num_entries   = ssh_mode_get_num_entries,
+    ._result            = ssh_mode_result,
+    ._destroy           = ssh_mode_destroy,
+    ._token_match       = ssh_token_match,
+    ._get_display_value = _get_display_value,
+    ._get_completion    = NULL,
+    ._preprocess_input  = NULL,
+    .private_data       = NULL,
+    .free               = NULL
 };
+/*@}*/
