@@ -3,7 +3,7 @@
  *
  * MIT/X11 License
  * Copyright (c) 2012 Sean Pringle <sean.pringle@gmail.com>
- * Modified 2013-2015 Qball Cow <qball@gmpclient.org>
+ * Modified 2013-2016 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -37,17 +37,22 @@
 #include <sys/types.h>
 #include <sys/file.h>
 #include <sys/stat.h>
+#include <pwd.h>
 #include <ctype.h>
+#include <xcb/xcb.h>
+#include <pango/pango.h>
+#include <pango/pango-fontmap.h>
+#include <pango/pangocairo.h>
 #include "helper.h"
+#include "settings.h"
 #include "x11-helper.h"
 #include "rofi.h"
+#include "view.h"
 
-static int  stored_argc   = 0;
-static char **stored_argv = NULL;
-
-// TODO: is this safe?
-#define NON_ASCII_NON_NULL( x )    ( ( ( x ) < 0 ) )
-#define ASCII_NON_NULL( x )        ( ( ( x ) > 0 ) )
+extern xcb_connection_t *xcb_connection;
+extern xcb_screen_t     *xcb_screen;
+static int              stored_argc   = 0;
+static char             **stored_argv = NULL;
 
 void cmd_set_arguments ( int argc, char **argv )
 {
@@ -55,32 +60,6 @@ void cmd_set_arguments ( int argc, char **argv )
     stored_argv = argv;
 }
 
-/**
- *  `fgets` implementation with custom separator.
- */
-char* fgets_s ( char* s, unsigned int n, FILE *iop, char sep )
-{
-    // Map these to registers.
-    register int c = EOF;
-    register char* cs;
-    cs = s;
-    // read until EOF or buffer is full.
-    while ( --n > 0 && ( c = getc ( iop ) ) != EOF ) {
-        // put the input char into the current pointer position, then increment it
-        // if a newline entered, break
-        if ( ( *cs++ = c ) == sep ) {
-            // Whipe separator
-            cs[-1] = '\0';
-            break;
-        }
-    }
-    // Always, 0 terminate the buffer.
-    *cs = '\0';
-    // if last read was end of file and current index is start, we are done:
-    // Return NULL.
-    return ( c == EOF && cs == s ) ? NULL : cs;
-}
-
 /**
  * @param info To Match information on.
  * @param res  The string being generated.
@@ -149,7 +128,7 @@ int helper_parse_setup ( char * string, char ***output, int *length, ... )
     // Throw error if shell parsing fails.
     if ( error ) {
         char *msg = g_strdup_printf ( "Failed to parse: '%s'\nError: '%s'", string, error->message );
-        error_dialog ( msg, FALSE );
+        rofi_view_error_dialog ( msg, FALSE );
         g_free ( msg );
         // print error.
         g_error_free ( error );
@@ -157,41 +136,71 @@ int helper_parse_setup ( char * string, char ***output, int *length, ... )
     return FALSE;
 }
 
-char *token_collate_key ( const char *token, int case_sensitive )
+void tokenize_free ( GRegex ** tokens )
 {
-    char *tmp, *compk;
-
-    if ( case_sensitive ) {
-        tmp = g_strdup ( token );
+    for ( size_t i = 0; tokens && tokens[i]; i++ ) {
+        g_regex_unref ( (GRegex *) tokens[i] );
     }
-    else {
-        tmp = g_utf8_casefold ( token, -1 );
-    }
-
-    compk = g_utf8_collate_key ( tmp, -1 );
-    g_free ( tmp );
-
-    return compk;
+    g_free ( tokens );
 }
 
-char **tokenize ( const char *input, int case_sensitive )
+static gchar *glob_to_regex ( const char *input )
+{
+    gchar  *r    = g_regex_escape_string ( input, -1 );
+    size_t str_l = strlen ( r );
+    for ( size_t i = 0; i < str_l; i++ ) {
+        if ( r[i] == '\\' ) {
+            if ( r[i + 1] == '*' ) {
+                r[i] = '.';
+            }
+            else if ( r[i + 1] == '?' ) {
+                r[i + 1] = 'S';
+            }
+            i++;
+        }
+    }
+    return r;
+}
+static GRegex * create_regex ( const char *input, int case_sensitive )
+{
+#define R( s )    g_regex_new ( s, G_REGEX_OPTIMIZE | ( ( case_sensitive ) ? 0 : G_REGEX_CASELESS ), 0, NULL )
+    GRegex * retv = NULL;
+    if ( config.glob ) {
+        gchar *r = glob_to_regex ( input );
+        retv = R ( r );
+        g_free ( r );
+    }
+    else if ( config.regex ) {
+        retv = R ( input );
+        if ( retv == NULL ) {
+            gchar *r = g_regex_escape_string ( input, -1 );
+            retv = R ( r );
+            g_free ( r );
+        }
+    }
+    else{
+        // TODO; regex should be default?
+        gchar *r = g_regex_escape_string ( input, -1 );
+        retv = R ( r );
+        g_free ( r );
+    }
+    return retv;
+}
+GRegex **tokenize ( const char *input, int case_sensitive )
 {
     if ( input == NULL ) {
         return NULL;
     }
+    size_t len = strlen ( input );
+    if ( len == 0 ) {
+        return NULL;
+    }
 
-    char *saveptr = NULL, *token;
-    char **retv   = NULL;
+    char   *saveptr = NULL, *token;
+    GRegex **retv   = NULL;
     if ( !config.tokenize ) {
-        retv = g_malloc0 ( sizeof ( char* ) * 2 );
-        if ( config.glob ) {
-            token                   = g_strconcat ( input, "*", NULL );
-            retv[0]                 = token_collate_key ( token, case_sensitive );
-            g_free ( token ); token = NULL;
-        }
-        else{
-            retv[0] = token_collate_key ( input, case_sensitive );
-        }
+        retv    = g_malloc0 ( sizeof ( GRegex* ) * 2 );
+        retv[0] = (GRegex *) create_regex ( input, case_sensitive );
         return retv;
     }
 
@@ -203,16 +212,10 @@ char **tokenize ( const char *input, int case_sensitive )
 
     // Iterate over tokens.
     // strtok should still be valid for utf8.
-    for ( token = strtok_r ( str, " ", &saveptr ); token != NULL; token = strtok_r ( NULL, " ", &saveptr ) ) {
-        retv = g_realloc ( retv, sizeof ( char* ) * ( num_tokens + 2 ) );
-        if ( config.glob ) {
-            char *t = token_collate_key ( token, case_sensitive );
-            retv[num_tokens] = g_strconcat ( t, "*", NULL );
-            g_free ( t );
-        }
-        else {
-            retv[num_tokens] = token_collate_key ( token, case_sensitive );
-        }
+    const char * const sep = " ";
+    for ( token = strtok_r ( str, sep, &saveptr ); token != NULL; token = strtok_r ( NULL, sep, &saveptr ) ) {
+        retv                 = g_realloc ( retv, sizeof ( GRegex* ) * ( num_tokens + 2 ) );
+        retv[num_tokens]     = (GRegex *) create_regex ( token, case_sensitive );
         retv[num_tokens + 1] = NULL;
         num_tokens++;
     }
@@ -266,59 +269,55 @@ int find_arg_uint ( const char * const key, unsigned int *val )
 
 char helper_parse_char ( const char *arg )
 {
-    char retv = -1;
-    int  len  = strlen ( arg );
+    int len = strlen ( arg );
     // If the length is 1, it is not escaped.
     if ( len == 1 ) {
-        retv = arg[0];
+        return arg[0];
     }
     // If the length is 2 and the first character is '\', we unescape it.
-    else if ( len == 2 && arg[0] == '\\' ) {
+    if ( len == 2 && arg[0] == '\\' ) {
         // New line
         if ( arg[1] == 'n' ) {
-            retv = '\n';
+            return '\n';
         }
         // Bell
         else if ( arg[1] == 'a' ) {
-            retv = '\a';
+            return '\a';
         }
         // Backspace
         else if ( arg[1] == 'b' ) {
-            retv = '\b';
+            return '\b';
         }
         // Tab
         else if ( arg[1] == 't' ) {
-            retv = '\t';
+            return '\t';
         }
         // Vertical tab
         else if ( arg[1] == 'v' ) {
-            retv = '\v';
+            return '\v';
         }
         // Form feed
         else if ( arg[1] == 'f' ) {
-            retv = '\f';
+            return '\f';
         }
         // Carriage return
         else if ( arg[1] == 'r' ) {
-            retv = '\r';
+            return '\r';
         }
         // Forward slash
         else if ( arg[1] == '\\' ) {
-            retv = '\\';
+            return '\\';
         }
         else if ( arg[1] == '0' ) {
-            retv = '\0';
+            return '\0';
         }
     }
-    else if ( len > 2 && arg[0] == '\\' && arg[1] == 'x' ) {
-        retv = (char) strtol ( &arg[2], NULL, 16 );
+    if ( len > 2 && arg[0] == '\\' && arg[1] == 'x' ) {
+        return (char) strtol ( &arg[2], NULL, 16 );
     }
-    if ( retv < 0 ) {
-        fprintf ( stderr, "Failed to parse character string: \"%s\"\n", arg );
-        // for now default to newline.
-        retv = '\n';
-    }
-    return retv;
+    fprintf ( stderr, "Failed to parse character string: \"%s\"\n", arg );
+    // for now default to newline.
+    return '\n';
 }
 
 int find_arg_char ( const char * const key, char *val )
@@ -332,135 +331,42 @@ int find_arg_char ( const char * const key, char *val )
     return FALSE;
 }
 
-/*
- * auxiliary to `fuzzy-token-match' below;
- */
-static void advance_unicode_glyph ( char** token_in, char** input_in )
+PangoAttrList *token_match_get_pango_attr ( GRegex **tokens, const char *input, PangoAttrList *retv )
 {
-    // determine the end of the glyph from token
-
-    char *token = *token_in;
-    char *input = *input_in;
-
-    while ( NON_ASCII_NON_NULL ( *token ) ) {
-        token++;
-    }
-
-    // now we know the glyph length, we can scan for that substring in input
-    // temporarily add a null-terminator in case:
-    char glyph_end = *token;
-    *token = 0;
-    char *match = strstr ( input, *token_in );
-    *token = glyph_end;
-
-    if ( match ) {
-        *token_in = token;
-        *input_in = match;
-    }
-    else {
-        // wind input along to the end so that we fail
-        while ( **input_in ) {
-            ( *input_in )++;
-        }
-    }
-}
-
-/**
- * Shared 'token_match' function.
- * Matches tokenized.
- */
-static int fuzzy_token_match ( char **tokens, const char *input, __attribute__( ( unused ) ) int not_ascii, int case_sensitive )
-{
-    int match = 1;
-
     // Do a tokenized match.
-
-    // TODO: this doesn't work for unicode input, because it may split a codepoint which is over two bytes.
-    //       mind you, it didn't work before I fiddled with it.
-
-    // this could perhaps be a bit more efficient by iterating over all the tokens at once.
-
     if ( tokens ) {
-        char *compk = not_ascii ? token_collate_key ( input, case_sensitive ) : (char *) input;
-        for ( int j = 0; match && tokens[j]; j++ ) {
-            char *t     = compk;
-            char *token = tokens[j];
-
-            while ( *t && *token ) {
-                if ( *token > 0 ) { // i.e. we are at an ascii codepoint
-                    if ( ( case_sensitive && ( *t == *token ) ) ||
-                         ( !case_sensitive && ( tolower ( *t ) == tolower ( *token ) ) ) ) {
-                        token++;
-                    }
-                }
-                else{
-                    // we are not at an ascii codepoint, and so we need to do something
-                    // complicated
-                    advance_unicode_glyph ( &token, &t );
-                }
-                t++;
+        for ( int j = 0; tokens[j]; j++ ) {
+            GMatchInfo *gmi = NULL;
+            g_regex_match ( (GRegex *) tokens[j], input, G_REGEX_MATCH_PARTIAL, &gmi );
+            while ( g_match_info_matches ( gmi ) ) {
+                int            start, end;
+                g_match_info_fetch_pos ( gmi, 0, &start, &end );
+                PangoAttribute *pa  = pango_attr_underline_new ( PANGO_UNDERLINE_SINGLE );
+                PangoAttribute *pa2 = pango_attr_weight_new ( PANGO_WEIGHT_BOLD );
+                pa2->start_index = pa->start_index = start;
+                pa2->end_index   = pa->end_index = end;
+                pango_attr_list_insert ( retv, pa );
+                pango_attr_list_insert ( retv, pa2 );
+                g_match_info_next ( gmi, NULL );
             }
-
-            match = !( *token );
-        }
-        if ( not_ascii ) {
-            g_free ( compk );
+            g_match_info_free ( gmi );
         }
     }
-
-    return match;
+    return retv;
 }
-static int normal_token_match ( char **tokens, const char *input, int not_ascii, int case_sensitive )
+
+int token_match ( GRegex **tokens, const char *input )
 {
     int match = 1;
-
-    // Do a tokenized match.
-
-    if ( tokens ) {
-        char *compk = not_ascii ? token_collate_key ( input, case_sensitive ) : (char *) input;
-        char *( *comparison )( const char *, const char * );
-        comparison = ( case_sensitive || not_ascii ) ? strstr : strcasestr;
-        for ( int j = 0; match && tokens[j]; j++ ) {
-            match = ( comparison ( compk, tokens[j] ) != NULL );
-        }
-        if ( not_ascii ) {
-            g_free ( compk );
-        }
-    }
-
-    return match;
-}
-
-static int glob_token_match ( char **tokens, const char *input, int not_ascii, int case_sensitive )
-{
-    int  match  = 1;
-    char *compk = not_ascii ? token_collate_key ( input, case_sensitive ) : (char *) input;
-
     // Do a tokenized match.
     if ( tokens ) {
         for ( int j = 0; match && tokens[j]; j++ ) {
-            match = g_pattern_match_simple (  tokens[j], compk );
+            match = g_regex_match ( (GRegex *) tokens[j], input, 0, NULL );
         }
     }
-    if ( not_ascii ) {
-        g_free ( compk );
-    }
     return match;
 }
 
-int token_match ( char **tokens, const char *input, int not_ascii, int case_sensitive,
-                  __attribute__( ( unused ) ) unsigned int index,
-                  __attribute__( ( unused ) ) Switcher *data )
-{
-    if ( config.glob ) {
-        return glob_token_match ( tokens, input, not_ascii, case_sensitive );
-    }
-    else if ( config.fuzzy ) {
-        return fuzzy_token_match ( tokens, input, not_ascii, case_sensitive );
-    }
-    return normal_token_match ( tokens, input, not_ascii, case_sensitive );
-}
-
 int execute_generator ( const char * cmd )
 {
     char **args = NULL;
@@ -473,7 +379,9 @@ int execute_generator ( const char * cmd )
 
     if ( error != NULL ) {
         char *msg = g_strdup_printf ( "Failed to execute: '%s'\nError: '%s'", cmd, error->message );
-        error_dialog ( msg, FALSE );
+        fputs ( msg, stderr );
+        fputs ( "\n", stderr );
+        rofi_view_error_dialog ( msg, FALSE );
         g_free ( msg );
         // print error.
         g_error_free ( error );
@@ -537,13 +445,8 @@ void remove_pid_file ( int fd )
  *
  * This functions exits the program with 1 when it finds an invalid configuration.
  */
-void config_sanity_check ( Display *display )
+int config_sanity_check ( void )
 {
-    // If alternative row is not set, copy the normal background color.
-    // Do this at the beginning as we might use it in the error dialog.
-    if ( config.menu_bg_alt == NULL ) {
-        config.menu_bg_alt = config.menu_bg;
-    }
     int     found_error = FALSE;
     GString *msg        = g_string_new (
         "<big><b>The configuration failed to validate:</b></big>\n" );
@@ -570,28 +473,13 @@ void config_sanity_check ( Display *display )
         config.location = WL_CENTER;
         found_error     = 1;
     }
-    if ( 0 ) {
-        if ( !( config.line_margin <= 50 ) ) {
-            g_string_append_printf ( msg, "\t<b>config.line_margin</b>=%d is invalid. Value should be between %d and %d.\n",
-                                     config.line_margin, 0, 50 );
-            config.line_margin = 2;
-            found_error        = 1;
-        }
-    }
-    if ( config.fullscreen && config.monitor != -1 ) {
-        g_string_append_printf ( msg,
-                                 "\t<b>config.monitor</b>=%d is invalid. Value should be unset (-1) when fullscreen mode is enabled.\n",
-                                 config.monitor );
-        config.monitor = -1;
-        found_error    = 1;
-    }
 
     // Check size
     {
-        int ssize = monitor_get_smallest_size ( display );
+        int ssize = monitor_get_smallest_size ( );
         if ( config.monitor >= 0 ) {
             workarea mon;
-            if ( monitor_get_dimension ( display, DefaultScreenOfDisplay ( display ), config.monitor, &mon ) ) {
+            if ( monitor_get_dimension ( config.monitor, &mon ) ) {
                 ssize = MIN ( mon.w, mon.h );
             }
             else{
@@ -601,7 +489,7 @@ void config_sanity_check ( Display *display )
         }
         // Have todo an estimate here.
         if ( ( 2 * ( config.padding + config.menu_bw ) ) > ( 0.9 * ssize ) ) {
-            g_string_append_printf ( msg, "\t<b>config.padding+config.menu_bw</b>=%d is to big for the minimum size of the monitor: %d.\n",
+            g_string_append_printf ( msg, "\t<b>config.padding+config.menu_bw</b>=%d is too big for the minimum size of the monitor: %d.\n",
                                      ( config.padding + config.menu_bw ), ssize );
 
             config.padding = 0;
@@ -610,22 +498,130 @@ void config_sanity_check ( Display *display )
         }
     }
 
+    if ( config.menu_font ) {
+        PangoFontDescription *pfd = pango_font_description_from_string ( config.menu_font );
+        const char           *fam = pango_font_description_get_family ( pfd );
+        int                  size = pango_font_description_get_size ( pfd );
+        if ( fam == NULL || size == 0 ) {
+            g_string_append_printf ( msg, "Pango failed to parse font: '%s'\n", config.menu_font );
+            g_string_append_printf ( msg, "Got font family: <b>%s</b> at size <b>%d</b>\n", fam ? fam : "{unknown}", size );
+            config.menu_font = NULL;
+            found_error      = TRUE;
+        }
+        pango_font_description_free ( pfd );
+    }
+
+    if ( config.monitor == -3 ) {
+        // On -3, set to location 1.
+        config.location   = 1;
+        config.fullscreen = 0;
+    }
+
     if ( found_error ) {
         g_string_append ( msg, "Please update your configuration." );
-        show_error_message ( msg->str, TRUE );
-        exit ( EXIT_FAILURE );
+        rofi_view_error_dialog ( msg->str, TRUE );
+        return TRUE;
     }
 
     g_string_free ( msg, TRUE );
+    return FALSE;
 }
 
-int is_not_ascii ( const char * str )
+char *rofi_expand_path ( const char *input )
 {
-    while ( ASCII_NON_NULL ( *str ) ) {
-        str++;
+    char **str = g_strsplit ( input, G_DIR_SEPARATOR_S, -1 );
+    for ( unsigned int i = 0; str && str[i]; i++ ) {
+        // Replace ~ with current user homedir.
+        if ( str[i][0] == '~' && str[i][1] == '\0' ) {
+            g_free ( str[i] );
+            str[i] = g_strdup ( g_get_home_dir () );
+        }
+        // If other user, ask getpwnam.
+        else if ( str[i][0] == '~' ) {
+            struct passwd *p = getpwnam ( &( str[i][1] ) );
+            if ( p != NULL ) {
+                g_free ( str[i] );
+                str[i] = g_strdup ( p->pw_dir );
+            }
+        }
+        else if ( i == 0 ) {
+            char * s = str[i];
+            if ( input[0] == G_DIR_SEPARATOR ) {
+                str[i] = g_strdup_printf ( "%s%s", G_DIR_SEPARATOR_S, s );
+                g_free ( s );
+            }
+        }
     }
-    if ( *str ) {
-        return 1;
+    char *retv = g_build_filenamev ( str );
+    g_strfreev ( str );
+    return retv;
+}
+
+#define MIN3( a, b, c )    ( ( a ) < ( b ) ? ( ( a ) < ( c ) ? ( a ) : ( c ) ) : ( ( b ) < ( c ) ? ( b ) : ( c ) ) )
+
+unsigned int levenshtein ( const char *needle, const char *haystack )
+{
+    unsigned int x, y, lastdiag, olddiag;
+    size_t       needlelen   = g_utf8_strlen ( needle, -1 );
+    size_t       haystacklen = g_utf8_strlen ( haystack, -1 );
+    unsigned int column[needlelen + 1];
+    for ( y = 0; y <= needlelen; y++ ) {
+        column[y] = y;
     }
-    return 0;
+    for ( x = 1; x <= haystacklen; x++ ) {
+        const char *needles = needle;
+        column[0] = x;
+        gunichar   haystackc = g_utf8_get_char ( haystack );
+        if ( !config.case_sensitive ) {
+            haystackc = g_unichar_tolower ( haystackc );
+        }
+        for ( y = 1, lastdiag = x - 1; y <= needlelen; y++ ) {
+            gunichar needlec = g_utf8_get_char ( needles );
+            if ( !config.case_sensitive ) {
+                needlec = g_unichar_tolower ( needlec );
+            }
+            olddiag   = column[y];
+            column[y] = MIN3 ( column[y] + 1, column[y - 1] + 1, lastdiag + ( needlec == haystackc ? 0 : 1 ) );
+            lastdiag  = olddiag;
+            needles   = g_utf8_next_char ( needles );
+        }
+        haystack = g_utf8_next_char ( haystack );
+    }
+    return column[needlelen];
+}
+
+char * rofi_latin_to_utf8_strdup ( const char *input, gssize length )
+{
+    gsize slength = 0;
+    return g_convert_with_fallback ( input, length, "UTF-8", "latin1", "\uFFFD", NULL, &slength, NULL );
+}
+
+char * rofi_force_utf8 ( gchar *start, ssize_t length )
+{
+    if ( start == NULL ) {
+        return NULL;
+    }
+    const char *data = start;
+    const char *end;
+    GString    *string;
+
+    if ( g_utf8_validate ( data, length, &end ) ) {
+        return g_memdup ( start, length + 1 );
+    }
+    string = g_string_sized_new ( length + 16 );
+
+    do {
+        /* Valid part of the string */
+        g_string_append_len ( string, data, end - data );
+        /* Replacement character */
+        g_string_append ( string, "\uFFFD" );
+        length -= ( end - data ) + 1;
+        data    = end + 1;
+    } while ( !g_utf8_validate ( data, length, &end ) );
+
+    if ( length ) {
+        g_string_append_len ( string, data, length );
+    }
+
+    return g_string_free ( string, FALSE );
 }
