@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2015 Qball Cow <qball@gmpclient.org>
+ * Copyright 2013-2016 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -35,14 +35,14 @@
 #include <glib/gstdio.h>
 #include "rofi.h"
 #include "history.h"
+#include "settings.h"
 
-#define HISTORY_NAME_LENGTH    256
 #define HISTORY_MAX_ENTRIES    25
 
 typedef struct __element
 {
     long int index;
-    char     name[HISTORY_NAME_LENGTH];
+    char     *name;
 }_element;
 
 static int __element_sort_func ( const void *ea, const void *eb, void *data __attribute__( ( unused ) ) )
@@ -74,7 +74,6 @@ static void __history_write_element_list ( FILE *fd, _element **list, unsigned i
 
 static _element ** __history_get_element_list ( FILE *fd, unsigned int *length )
 {
-    char     buffer[HISTORY_NAME_LENGTH + 16];
     _element **retv = NULL;
 
     if  ( length == NULL ) {
@@ -85,27 +84,43 @@ static _element ** __history_get_element_list ( FILE *fd, unsigned int *length )
     if ( fd == NULL ) {
         return NULL;
     }
-    while ( fgets ( buffer, HISTORY_NAME_LENGTH + 16, fd ) != NULL ) {
+    char    *buffer       = NULL;
+    size_t  buffer_length = 0;
+    ssize_t l             = 0;
+    while ( ( l = getline ( &buffer, &buffer_length, fd ) ) > 0 ) {
         char * start = NULL;
         // Skip empty lines.
-        if ( strlen ( buffer ) == 0 ) {
+        if ( l <= 1 ) {
+            continue;
+        }
+
+        long int index = strtol ( buffer, &start, 10 );
+        if ( start == buffer || *start == '\0' ) {
+            continue;
+        }
+        start++;
+        if ( ( l - ( start - buffer ) ) < 2 ) {
             continue;
         }
         // Resize and check.
         retv = g_realloc ( retv, ( *length + 2 ) * sizeof ( _element* ) );
 
         retv[( *length )] = g_malloc ( sizeof ( _element ) );
+
         // remove trailing \n
-        buffer[strlen ( buffer ) - 1] = '\0';
+        buffer[l - 1] = '\0';
         // Parse the number of times.
-        retv[( *length )]->index = strtol ( buffer, &start, 10 );
-        strncpy ( retv[( *length )]->name, ( start + 1 ), HISTORY_NAME_LENGTH );
+        retv[( *length )]->index = index;
+        retv[( *length )]->name  = g_strndup ( start, l - 1 - ( start - buffer ) );
         // Force trailing '\0'
-        retv[( *length )]->name[HISTORY_NAME_LENGTH - 1] = '\0';
-        retv[( *length ) + 1]                            = NULL;
+        retv[( *length ) + 1] = NULL;
 
         ( *length )++;
     }
+    if ( buffer != NULL  ) {
+        free ( buffer );
+        buffer = NULL;
+    }
     return retv;
 }
 
@@ -119,14 +134,15 @@ void history_set ( const char *filename, const char *entry )
     unsigned int length = 0;
     _element     **list = NULL;
     // Open file for reading and writing.
-    FILE         *fd = g_fopen ( filename, "a+" );
-    if ( fd == NULL ) {
-        fprintf ( stderr, "Failed to open file: %s\n", strerror ( errno ) );
-        return;
+    FILE         *fd = g_fopen ( filename, "r" );
+    if ( fd != NULL ) {
+        // Get list.
+        list = __history_get_element_list ( fd, &length );
+        // Close file, if fails let user know on stderr.
+        if ( fclose ( fd ) != 0 ) {
+            fprintf ( stderr, "Failed to close history file: %s\n", strerror ( errno ) );
+        }
     }
-    // Get list.
-    list = __history_get_element_list ( fd, &length );
-
     // Look if the entry exists.
     for ( unsigned int iter = 0; !found && iter < length; iter++ ) {
         if ( strcmp ( list[iter]->name, entry ) == 0 ) {
@@ -146,8 +162,7 @@ void history_set ( const char *filename, const char *entry )
         list[length] = g_malloc ( sizeof ( _element ) );
         // Copy name
         if ( list[length] != NULL ) {
-            g_strlcpy ( list[length]->name, entry, HISTORY_NAME_LENGTH );
-            list[length]->name[HISTORY_NAME_LENGTH - 1] = '\0';
+            list[length]->name = g_strdup ( entry );
             // set # hits
             list[length]->index = 1;
 
@@ -156,25 +171,24 @@ void history_set ( const char *filename, const char *entry )
         }
     }
 
-    // Rewind.
-    fseek ( fd, 0L, SEEK_SET );
-    // Clear file.
-    if ( ftruncate ( fileno ( fd ), 0 ) == 0 ) {
+    fd = fopen ( filename, "w" );
+    if ( fd == NULL ) {
+        fprintf ( stderr, "Failed to open file: %s\n", strerror ( errno ) );
+    }
+    else {
         // Write list.
         __history_write_element_list ( fd, list, length );
-    }
-    else{
-        fprintf ( stderr, "Failed to truncate file: %s\n", strerror ( errno ) );
+        // Close file, if fails let user know on stderr.
+        if ( fclose ( fd ) != 0 ) {
+            fprintf ( stderr, "Failed to close history file: %s\n", strerror ( errno ) );
+        }
     }
     // Free the list.
     for ( unsigned int iter = 0; iter < length; iter++ ) {
+        g_free ( list[iter]->name );
         g_free ( list[iter] );
     }
     g_free ( list );
-    // Close file, if fails let user know on stderr.
-    if ( fclose ( fd ) != 0 ) {
-        fprintf ( stderr, "Failed to close history file: %s\n", strerror ( errno ) );
-    }
 }
 
 void history_remove ( const char *filename, const char *entry )
@@ -187,7 +201,7 @@ void history_remove ( const char *filename, const char *entry )
     unsigned int curr    = 0;
     unsigned int length  = 0;
     // Open file for reading and writing.
-    FILE         *fd = g_fopen ( filename, "a+" );
+    FILE         *fd = g_fopen ( filename, "r" );
     if ( fd == NULL ) {
         fprintf ( stderr, "Failed to open file: %s\n", strerror ( errno ) );
         return;
@@ -195,6 +209,10 @@ void history_remove ( const char *filename, const char *entry )
     // Get list.
     list = __history_get_element_list ( fd, &length );
 
+    // Close file, if fails let user know on stderr.
+    if ( fclose ( fd ) != 0 ) {
+        fprintf ( stderr, "Failed to close history file: %s\n", strerror ( errno ) );
+    }
     // Find entry.
     for ( unsigned int iter = 0; !found && iter < length; iter++ ) {
         if ( strcmp ( list[iter]->name, entry ) == 0 ) {
@@ -206,6 +224,7 @@ void history_remove ( const char *filename, const char *entry )
     // If found, remove it and write out new file.
     if ( found ) {
         // Remove the entry.
+        g_free ( list[curr]->name );
         g_free ( list[curr] );
         // Swap last to here (if list is size 1, we just swap empty sets).
         list[curr] = list[length - 1];
@@ -213,12 +232,15 @@ void history_remove ( const char *filename, const char *entry )
         list[length - 1] = NULL;
         length--;
 
-        // Rewind.
-        fseek ( fd, 0L, SEEK_SET );
+        fd = g_fopen ( filename, "w" );
         // Clear list.
-        if ( ftruncate ( fileno ( fd ), 0 ) == 0 ) {
+        if ( fd != NULL ) {
             // Write list.
             __history_write_element_list ( fd, list, length );
+            // Close file, if fails let user know on stderr.
+            if ( fclose ( fd ) != 0 ) {
+                fprintf ( stderr, "Failed to close history file: %s\n", strerror ( errno ) );
+            }
         }
         else{
             fprintf ( stderr, "Failed to open file: %s\n", strerror ( errno ) );
@@ -227,16 +249,12 @@ void history_remove ( const char *filename, const char *entry )
 
     // Free the list.
     for ( unsigned int iter = 0; iter < length; iter++ ) {
+        g_free ( list[iter]->name );
         g_free ( list[iter] );
     }
     if ( list != NULL ) {
         g_free ( list );
     }
-
-    // Close file, if fails let user know on stderr.
-    if ( fclose ( fd ) != 0 ) {
-        fprintf ( stderr, "Failed to close history file: %s\n", strerror ( errno ) );
-    }
 }
 
 char ** history_get_list ( const char *filename, unsigned int *length )
@@ -266,7 +284,7 @@ char ** history_get_list ( const char *filename, unsigned int *length )
     if ( ( *length ) > 0 ) {
         retv = g_malloc ( ( ( *length ) + 1 ) * sizeof ( char * ) );
         for ( unsigned int iter = 0; iter < ( *length ); iter++ ) {
-            retv[iter] = g_strdup ( list[iter]->name );
+            retv[iter] = ( list[iter]->name );
             g_free ( list[iter] );
         }
         retv[( *length )] = NULL;
