@@ -1,7 +1,7 @@
 /**
  *   MIT/X11 License
  *   Copyright (c) 2012 Sean Pringle <sean.pringle@gmail.com>
- *   Modified  (c) 2013-2015 Qball Cow <qball@gmpclient.org>
+ *   Modified  (c) 2013-2016 Qball Cow <qball@gmpclient.org>
  *
  *   Permission is hereby granted, free of charge, to any person obtaining
  *   a copy of this software and associated documentation files (the
@@ -24,29 +24,27 @@
  */
 
 #include <config.h>
-#include <X11/X.h>
-#include <X11/Xatom.h>
-#include <X11/Xlib.h>
-#include <X11/Xmd.h>
-#include <X11/Xutil.h>
-#include <X11/Xproto.h>
-#include <X11/keysym.h>
-#include <X11/XKBlib.h>
+#include <xcb/xcb.h>
 #include <ctype.h>
+#include <string.h>
 #include <glib.h>
-#include <pango/pangocairo.h>
-#include "rofi.h"
+#include <math.h>
+#include "settings.h"
 #include "textbox.h"
 #include "keyb.h"
 #include "x11-helper.h"
+#include "mode.h"
+#include "view.h"
 
 #define SIDE_MARGIN    1
 
+#define DOT_OFFSET     15
+
 /**
  * Font + font color cache.
  * Avoid re-loading font on every change on every textbox.
  */
-typedef struct _RowColor
+typedef struct
 {
     Color fg;
     Color bg;
@@ -56,30 +54,41 @@ typedef struct _RowColor
 } RowColor;
 
 #define num_states    3
-RowColor     colors[num_states];
+RowColor                colors[num_states];
 
-PangoContext *p_context = NULL;
+static PangoContext     *p_context = NULL;
+static PangoFontMetrics *p_metrics = NULL;
+static gboolean textbox_blink ( gpointer data )
+{
+    textbox *tb = (textbox *) data;
+    if ( tb->blink < 2 ) {
+        tb->blink  = !tb->blink;
+        tb->update = TRUE;
+        rofi_view_queue_redraw ( );
+    }
+    else {
+        tb->blink--;
+    }
+    return TRUE;
+}
 
 textbox* textbox_create ( TextboxFlags flags, short x, short y, short w, short h,
                           TextBoxFontType tbft, const char *text )
 {
-    textbox *tb = g_malloc0 ( sizeof ( textbox ) );
+    textbox *tb = g_slice_new0 ( textbox );
 
     tb->flags = flags;
 
-    tb->x = x;
-    tb->y = y;
-    tb->w = MAX ( 1, w );
-    tb->h = MAX ( 1, h );
+    tb->widget.x = x;
+    tb->widget.y = y;
+    tb->widget.w = MAX ( 1, w );
+    tb->widget.h = MAX ( 1, h );
 
     tb->changed = FALSE;
 
-    tb->main_surface = cairo_image_surface_create ( get_format (), tb->w, tb->h );
+    tb->main_surface = cairo_image_surface_create ( CAIRO_FORMAT_ARGB32, tb->widget.w, tb->widget.h );
     tb->main_draw    = cairo_create ( tb->main_surface );
-    tb->layout       = pango_cairo_create_layout ( tb->main_draw );
-    PangoFontDescription *pfd = pango_font_description_from_string ( config.menu_font );
-    pango_layout_set_font_description ( tb->layout, pfd );
-    pango_font_description_free ( pfd );
+    tb->layout       = pango_layout_new ( p_context );
     textbox_font ( tb, tbft );
 
     if ( ( flags & TB_WRAP ) == TB_WRAP ) {
@@ -89,8 +98,16 @@ textbox* textbox_create ( TextboxFlags flags, short x, short y, short w, short h
     textbox_cursor_end ( tb );
 
     // auto height/width modes get handled here
-    textbox_moveresize ( tb, tb->x, tb->y, tb->w, tb->h );
+    textbox_moveresize ( tb, tb->widget.x, tb->widget.y, tb->widget.w, tb->widget.h );
 
+    tb->blink_timeout = 0;
+    tb->blink         = 1;
+    if ( ( flags & TB_EDITABLE ) == TB_EDITABLE ) {
+        tb->blink_timeout = g_timeout_add ( 1200, textbox_blink, tb );
+    }
+
+    // Enabled by default
+    tb->widget.enabled = TRUE;
     return tb;
 }
 
@@ -123,6 +140,43 @@ void textbox_font ( textbox *tb, TextBoxFontType tbft )
     tb->tbft = tbft;
 }
 
+/**
+ * @param tb The textbox object.
+ *
+ * Update the pango layout's text. It does this depending on the
+ * textbox flags.
+ */
+static void __textbox_update_pango_text ( textbox *tb )
+{
+    if ( ( tb->flags & TB_PASSWORD ) == TB_PASSWORD ) {
+        size_t l = g_utf8_strlen ( tb->text, -1 );
+        char   string [l + 1];
+        memset ( string, '*', l );
+        string[l] = '\0';
+        pango_layout_set_attributes ( tb->layout, NULL );
+        pango_layout_set_text ( tb->layout, string, l );
+    }
+    else if ( tb->flags & TB_MARKUP || tb->tbft & MARKUP ) {
+        pango_layout_set_markup ( tb->layout, tb->text, -1 );
+    }
+    else {
+        pango_layout_set_attributes ( tb->layout, NULL );
+        pango_layout_set_text ( tb->layout, tb->text, -1 );
+    }
+}
+const char *textbox_get_visible_text ( textbox *tb )
+{
+    return pango_layout_get_text ( tb->layout );
+}
+PangoAttrList *textbox_get_pango_attributes ( textbox *tb )
+{
+    return pango_layout_get_attributes ( tb->layout );
+}
+void textbox_set_pango_attributes ( textbox *tb, PangoAttrList *list )
+{
+    pango_layout_set_attributes ( tb->layout, list );
+}
+
 // set the default text to display
 void textbox_text ( textbox *tb, const char *text )
 {
@@ -142,30 +196,18 @@ void textbox_text ( textbox *tb, const char *text )
             tb->text = g_strdup ( "Invalid UTF-8 string." );
         }
     }
-
-    if ( tb->flags & TB_MARKUP || tb->tbft & MARKUP ) {
-        pango_layout_set_markup ( tb->layout, tb->text, strlen ( tb->text ) );
-    }
-    else {
-        pango_layout_set_text ( tb->layout, tb->text, strlen ( tb->text ) );
-    }
+    __textbox_update_pango_text ( tb );
     if ( tb->flags & TB_AUTOWIDTH ) {
-        textbox_moveresize ( tb, tb->x, tb->y, tb->w, tb->h );
+        textbox_moveresize ( tb, tb->widget.x, tb->widget.y, tb->widget.w, tb->widget.h );
     }
 
     tb->cursor = MAX ( 0, MIN ( ( int ) strlen ( text ), tb->cursor ) );
 }
 
-void textbox_move ( textbox *tb, int x, int y )
-{
-    if ( x != tb->x || y != tb->y ) {
-        tb->x = x;
-        tb->y = y;
-    }
-}
 // within the parent handled auto width/height modes
 void textbox_moveresize ( textbox *tb, int x, int y, int w, int h )
 {
+    unsigned int offset = ( tb->flags & TB_INDICATOR ) ? DOT_OFFSET : 0;
     if ( tb->flags & TB_AUTOWIDTH ) {
         pango_layout_set_width ( tb->layout, -1 );
         w = textbox_get_width ( tb );
@@ -183,19 +225,19 @@ void textbox_moveresize ( textbox *tb, int x, int y, int w, int h )
     if ( tb->flags & TB_AUTOHEIGHT ) {
         // Width determines height!
         int tw = MAX ( 1, w );
-        pango_layout_set_width ( tb->layout, PANGO_SCALE * ( tw - 2 * SIDE_MARGIN ) );
+        pango_layout_set_width ( tb->layout, PANGO_SCALE * ( tw - 2 * SIDE_MARGIN - offset ) );
         h = textbox_get_height ( tb );
     }
 
-    if ( x != tb->x || y != tb->y || w != tb->w || h != tb->h ) {
-        tb->x = x;
-        tb->y = y;
-        tb->h = MAX ( 1, h );
-        tb->w = MAX ( 1, w );
+    if ( x != tb->widget.x || y != tb->widget.y || w != tb->widget.w || h != tb->widget.h ) {
+        tb->widget.x = x;
+        tb->widget.y = y;
+        tb->widget.h = MAX ( 1, h );
+        tb->widget.w = MAX ( 1, w );
     }
 
     // We always want to update this
-    pango_layout_set_width ( tb->layout, PANGO_SCALE * ( tb->w - 2 * SIDE_MARGIN ) );
+    pango_layout_set_width ( tb->layout, PANGO_SCALE * ( tb->widget.w - 2 * SIDE_MARGIN - offset ) );
     tb->update = TRUE;
 }
 
@@ -205,6 +247,10 @@ void textbox_free ( textbox *tb )
     if ( tb == NULL ) {
         return;
     }
+    if ( tb->blink_timeout > 0 ) {
+        g_source_remove ( tb->blink_timeout );
+        tb->blink_timeout = 0;
+    }
 
     g_free ( tb->text );
 
@@ -220,11 +266,12 @@ void textbox_free ( textbox *tb )
         tb->main_surface = NULL;
     }
 
-    g_free ( tb );
+    g_slice_free ( textbox, tb );
 }
 
 static void texbox_update ( textbox *tb )
 {
+    unsigned int offset = ( tb->flags & TB_INDICATOR ) ? DOT_OFFSET : 0;
     if ( tb->update ) {
         if ( tb->main_surface ) {
             cairo_destroy ( tb->main_draw );
@@ -232,10 +279,8 @@ static void texbox_update ( textbox *tb )
             tb->main_draw    = NULL;
             tb->main_surface = NULL;
         }
-        tb->main_surface = cairo_image_surface_create ( get_format (), tb->w, tb->h );
+        tb->main_surface = cairo_image_surface_create ( CAIRO_FORMAT_ARGB32, tb->widget.w, tb->widget.h );
         tb->main_draw    = cairo_create ( tb->main_surface );
-        PangoFontDescription *pfd = pango_font_description_from_string ( config.menu_font );
-        pango_font_description_free ( pfd );
         cairo_set_operator ( tb->main_draw, CAIRO_OPERATOR_SOURCE );
 
         pango_cairo_update_layout ( tb->main_draw, tb->layout );
@@ -247,12 +292,7 @@ static void texbox_update ( textbox *tb )
         int  cursor_width = MAX ( 2, font_height / 10 );
 
         if ( tb->changed ) {
-            if ( tb->flags & TB_MARKUP ) {
-                pango_layout_set_markup ( tb->layout, text, text_len );
-            }
-            else{
-                pango_layout_set_text ( tb->layout, text, text_len );
-            }
+            __textbox_update_pango_text ( tb );
         }
 
         if ( tb->flags & TB_EDITABLE ) {
@@ -265,42 +305,49 @@ static void texbox_update ( textbox *tb )
         }
 
         // Skip the side MARGIN on the X axis.
-        int x = SIDE_MARGIN;
+        int x = SIDE_MARGIN + offset;
         int y = 0;
 
         if ( tb->flags & TB_RIGHT ) {
             int line_width = 0;
             // Get actual width.
             pango_layout_get_pixel_size ( tb->layout, &line_width, NULL );
-            x = ( tb->w - line_width - SIDE_MARGIN );
+            x = ( tb->widget.w - line_width - SIDE_MARGIN - offset );
         }
         else if ( tb->flags & TB_CENTER ) {
             int tw = textbox_get_font_width ( tb );
-            x = (  ( tb->w - tw - 2 * SIDE_MARGIN ) ) / 2;
-        }
-        short fh = textbox_get_font_height ( tb );
-        if ( fh > tb->h ) {
-            y = 0;
-        }
-        else {
-            y = (   ( tb->h - fh ) ) / 2;
+            x = (  ( tb->widget.w - tw - 2 * SIDE_MARGIN - offset ) ) / 2;
         }
+        y = SIDE_MARGIN + ( pango_font_metrics_get_ascent ( p_metrics ) - pango_layout_get_baseline ( tb->layout ) ) / PANGO_SCALE;
 
         // Set ARGB
         Color col = tb->color_bg;
         cairo_set_source_rgba ( tb->main_draw, col.red, col.green, col.blue, col.alpha );
         cairo_paint ( tb->main_draw );
 
-        // Set ARGB
         col = tb->color_fg;
         cairo_set_source_rgba ( tb->main_draw, col.red, col.green, col.blue, col.alpha );
+        // draw the cursor
+        if ( tb->flags & TB_EDITABLE && tb->blink ) {
+            cairo_rectangle ( tb->main_draw, x + cursor_x, y, cursor_width, font_height );
+            cairo_fill ( tb->main_draw );
+        }
+
+        // Set ARGB
+        // We need to set over, otherwise subpixel hinting wont work.
+        cairo_set_operator ( tb->main_draw, CAIRO_OPERATOR_OVER );
         cairo_move_to ( tb->main_draw, x, y );
         pango_cairo_show_layout ( tb->main_draw, tb->layout );
 
-        //cairo_fill(tb->draw);
-        // draw the cursor
-        if ( tb->flags & TB_EDITABLE ) {
-            cairo_rectangle ( tb->main_draw, x + cursor_x, y, cursor_width, font_height );
+        if ( ( tb->flags & TB_INDICATOR ) == TB_INDICATOR && ( tb->tbft & ( SELECTED | HIGHLIGHT ) ) ) {
+            if ( ( tb->tbft & SELECTED ) == SELECTED ) {
+                cairo_set_source_rgba ( tb->main_draw, col.red, col.green, col.blue, col.alpha );
+            }
+            else if ( ( tb->tbft & HIGHLIGHT ) == HIGHLIGHT ) {
+                cairo_set_source_rgba ( tb->main_draw, col.red, col.green, col.blue, col.alpha * 0.2 );
+            }
+
+            cairo_arc ( tb->main_draw, DOT_OFFSET / 2.0, tb->widget.h / 2.0, 2.0, 0, 2.0 * M_PI );
             cairo_fill ( tb->main_draw );
         }
 
@@ -309,12 +356,15 @@ static void texbox_update ( textbox *tb )
 }
 void textbox_draw ( textbox *tb, cairo_t *draw )
 {
+    if ( tb->widget.enabled == FALSE ) {
+        return;
+    }
     texbox_update ( tb );
 
     /* Write buffer */
 
-    cairo_set_source_surface ( draw, tb->main_surface, tb->x, tb->y );
-    cairo_rectangle ( draw, tb->x, tb->y, tb->w, tb->h );
+    cairo_set_source_surface ( draw, tb->main_surface, tb->widget.x, tb->widget.y );
+    cairo_rectangle ( draw, tb->widget.x, tb->widget.y, tb->widget.w, tb->widget.h );
     cairo_fill ( draw );
 }
 
@@ -405,14 +455,19 @@ static void textbox_cursor_dec_word ( textbox *tb )
 // end of line
 void textbox_cursor_end ( textbox *tb )
 {
+    if ( tb->text == NULL ) {
+        tb->cursor = 0;
+        tb->update = TRUE;
+        return;
+    }
     tb->cursor = ( int ) strlen ( tb->text );
     tb->update = TRUE;
 }
 
 // insert text
-void textbox_insert ( textbox *tb, int pos, char *str )
+void textbox_insert ( textbox *tb, int pos, char *str, int slen )
 {
-    int len = ( int ) strlen ( tb->text ), slen = ( int ) strlen ( str );
+    int len = ( int ) strlen ( tb->text );
     pos = MAX ( 0, MIN ( len, pos ) );
     // expand buffer
     tb->text = g_realloc ( tb->text, len + slen + 1 );
@@ -475,6 +530,24 @@ static void textbox_cursor_bkspc_word ( textbox *tb )
         }
     }
 }
+static void textbox_cursor_del_eol ( textbox *tb )
+{
+    if ( tb->cursor >= 0 ) {
+        int length = strlen ( tb->text ) - tb->cursor;
+        if ( length >= 0 ) {
+            textbox_delete ( tb, tb->cursor, length );
+        }
+    }
+}
+static void textbox_cursor_del_sol ( textbox *tb )
+{
+    if ( tb->cursor >= 0 ) {
+        int length = tb->cursor;
+        if ( length >= 0 ) {
+            textbox_delete ( tb, 0, length );
+        }
+    }
+}
 static void textbox_cursor_del_word ( textbox *tb )
 {
     if ( tb->cursor >= 0 ) {
@@ -491,171 +564,141 @@ static void textbox_cursor_del_word ( textbox *tb )
 // 0 = unhandled
 // 1 = handled
 // -1 = handled and return pressed (finished)
-int textbox_keypress ( textbox *tb, XIC xic, XEvent *ev )
+int textbox_keybinding ( textbox *tb, KeyBindingAction action )
 {
-    KeySym key;
-    Status stat;
-    char   pad[32];
-    int    len;
-
     if ( !( tb->flags & TB_EDITABLE ) ) {
-        return 0;
+        g_return_val_if_reached ( 0 );
     }
 
-    len      = Xutf8LookupString ( xic, &ev->xkey, pad, sizeof ( pad ), &key, &stat );
-    pad[len] = 0;
+    switch ( action )
+    {
     // Left or Ctrl-b
-    if ( abe_test_action ( MOVE_CHAR_BACK, ev->xkey.state, key ) ) {
+    case MOVE_CHAR_BACK:
         textbox_cursor_dec ( tb );
         return 2;
-    }
     // Right or Ctrl-F
-    else if ( abe_test_action ( MOVE_CHAR_FORWARD, ev->xkey.state, key ) ) {
+    case MOVE_CHAR_FORWARD:
         textbox_cursor_inc ( tb );
         return 2;
-    }
-
     // Ctrl-U: Kill from the beginning to the end of the line.
-    else if ( abe_test_action ( CLEAR_LINE, ev->xkey.state, key ) ) {
+    case CLEAR_LINE:
         textbox_text ( tb, "" );
         return 1;
-    }
     // Ctrl-A
-    else if ( abe_test_action ( MOVE_FRONT, ev->xkey.state, key ) ) {
+    case MOVE_FRONT:
         textbox_cursor ( tb, 0 );
         return 2;
-    }
     // Ctrl-E
-    else if ( abe_test_action ( MOVE_END, ev->xkey.state, key ) ) {
+    case MOVE_END:
         textbox_cursor_end ( tb );
         return 2;
-    }
     // Ctrl-Alt-h
-    else if ( abe_test_action ( REMOVE_WORD_BACK, ev->xkey.state, key ) ) {
+    case REMOVE_WORD_BACK:
         textbox_cursor_bkspc_word ( tb );
         return 1;
-    }
     // Ctrl-Alt-d
-    else if ( abe_test_action ( REMOVE_WORD_FORWARD, ev->xkey.state, key ) ) {
+    case REMOVE_WORD_FORWARD:
         textbox_cursor_del_word ( tb );
         return 1;
-    }    // Delete or Ctrl-D
-    else if ( abe_test_action ( REMOVE_CHAR_FORWARD, ev->xkey.state, key ) ) {
+    case REMOVE_TO_EOL:
+        textbox_cursor_del_eol ( tb );
+        return 1;
+    case REMOVE_TO_SOL:
+        textbox_cursor_del_sol ( tb );
+        return 1;
+    // Delete or Ctrl-D
+    case REMOVE_CHAR_FORWARD:
         textbox_cursor_del ( tb );
         return 1;
-    }
     // Alt-B
-    else if ( abe_test_action ( MOVE_WORD_BACK, ev->xkey.state, key ) ) {
+    case MOVE_WORD_BACK:
         textbox_cursor_dec_word ( tb );
         return 2;
-    }
     // Alt-F
-    else if ( abe_test_action ( MOVE_WORD_FORWARD, ev->xkey.state, key ) ) {
+    case MOVE_WORD_FORWARD:
         textbox_cursor_inc_word ( tb );
         return 2;
-    }
     // BackSpace, Ctrl-h
-    else if ( abe_test_action ( REMOVE_CHAR_BACK, ev->xkey.state, key ) ) {
+    case REMOVE_CHAR_BACK:
         textbox_cursor_bkspc ( tb );
         return 1;
+    default:
+        g_return_val_if_reached ( 0 );
     }
-    else if ( abe_test_action ( ACCEPT_CUSTOM, ev->xkey.state, key ) ) {
-        return -2;
-    }
-    else if  ( abe_test_action ( ACCEPT_ENTRY_CONTINUE, ev->xkey.state, key ) ) {
-        return -3;
-    }
-    else if ( abe_test_action ( ACCEPT_ENTRY, ev->xkey.state, key ) ) {
-        return -1;
-    }
-    else if ( !iscntrl ( *pad ) ) {
-        textbox_insert ( tb, tb->cursor, pad );
-        textbox_cursor_inc ( tb );
-        return 1;
+}
+
+gboolean textbox_append_char ( textbox *tb, char *pad, int pad_len )
+{
+    if ( !( tb->flags & TB_EDITABLE ) ) {
+        return FALSE;
     }
 
-    return 0;
+    // Filter When alt/ctrl is pressed do not accept the character.
+    if ( !g_unichar_iscntrl ( g_utf8_get_char ( pad ) ) ) {
+        tb->blink = 2;
+        textbox_insert ( tb, tb->cursor, pad, pad_len );
+        textbox_cursor ( tb, tb->cursor + pad_len );
+        return TRUE;
+    }
+    return FALSE;
 }
 
 /***
  * Font setup.
  */
-static void parse_color ( Display *display, char *bg, Color *col )
-{
-    unsigned int val = 0;
-    val        = color_get ( display, bg, "white" );
-    col->alpha = ( ( val & 0xFF000000 ) >> 24 ) / 256.0;
-    col->red   = ( ( val & 0x00FF0000 ) >> 16 ) / 256.0;
-    col->green = ( ( val & 0x0000FF00 ) >> 8  ) / 256.0;
-    col->blue  = ( ( val & 0x000000FF )       ) / 256.0;
-}
-static void textbox_parse_string (  Display *display, const char *str, RowColor *color )
+static void textbox_parse_string (  const char *str, RowColor *color )
 {
     if ( str == NULL ) {
         return;
     }
-    char *cstr = g_strdup ( str );
-    char *endp = NULL;
-    char *token;
-    int  index = 0;
-    for ( token = strtok_r ( cstr, ",", &endp ); token != NULL; token = strtok_r ( NULL, ",", &endp ) ) {
+    char              *cstr = g_strdup ( str );
+    char              *endp = NULL;
+    char              *token;
+    int               index = 0;
+    const char *const sep   = ",";
+    for ( token = strtok_r ( cstr, sep, &endp ); token != NULL; token = strtok_r ( NULL, sep, &endp ) ) {
         switch ( index )
         {
         case 0:
-            parse_color ( display, g_strstrip ( token ), &( color->bg ) );
+            color->bg = color_get ( g_strstrip ( token ) );
             break;
         case 1:
-            parse_color ( display, g_strstrip ( token ), &( color->fg ) );
+            color->fg = color_get ( g_strstrip ( token ) );
             break;
         case 2:
-            parse_color ( display, g_strstrip ( token ), &( color->bgalt ) );
+            color->bgalt = color_get ( g_strstrip ( token ) );
             break;
         case 3:
-            parse_color ( display, g_strstrip ( token ), &( color->hlbg ) );
+            color->hlbg = color_get ( g_strstrip ( token ) );
             break;
         case 4:
-            parse_color ( display, g_strstrip ( token ), &( color->hlfg ) );
+            color->hlfg = color_get ( g_strstrip ( token ) );
             break;
         }
         index++;
     }
     g_free ( cstr );
 }
-void textbox_setup ( Display *display )
+void textbox_setup ( void )
 {
-    if ( config.color_enabled ) {
-        textbox_parse_string ( display, config.color_normal, &( colors[NORMAL] ) );
-        textbox_parse_string ( display, config.color_urgent, &( colors[URGENT] ) );
-        textbox_parse_string ( display, config.color_active, &( colors[ACTIVE] ) );
-    }
-    else {
-        parse_color ( display, config.menu_bg, &( colors[NORMAL].bg ) );
-        parse_color ( display, config.menu_fg, &( colors[NORMAL].fg ) );
-        parse_color ( display, config.menu_bg_alt, &( colors[NORMAL].bgalt ) );
-        parse_color ( display, config.menu_hlfg, &( colors[NORMAL].hlfg ) );
-        parse_color ( display, config.menu_hlbg, &( colors[NORMAL].hlbg ) );
+    textbox_parse_string ( config.color_normal, &( colors[NORMAL] ) );
+    textbox_parse_string ( config.color_urgent, &( colors[URGENT] ) );
+    textbox_parse_string ( config.color_active, &( colors[ACTIVE] ) );
+}
 
-        parse_color ( display, config.menu_bg_urgent, &( colors[URGENT].bg ) );
-        parse_color ( display, config.menu_fg_urgent, &( colors[URGENT].fg ) );
-        parse_color ( display, config.menu_bg_alt, &( colors[URGENT].bgalt ) );
-        parse_color ( display, config.menu_hlfg_urgent, &( colors[URGENT].hlfg ) );
-        parse_color ( display, config.menu_hlbg_urgent, &( colors[URGENT].hlbg ) );
-
-        parse_color ( display, config.menu_bg_active, &( colors[ACTIVE].bg ) );
-        parse_color ( display, config.menu_fg_active, &( colors[ACTIVE].fg ) );
-        parse_color ( display, config.menu_bg_alt, &( colors[ACTIVE].bgalt ) );
-        parse_color ( display, config.menu_hlfg_active, &( colors[ACTIVE].hlfg ) );
-        parse_color ( display, config.menu_hlbg_active, &( colors[ACTIVE].hlbg ) );
-    }
-    PangoFontMap *font_map = pango_cairo_font_map_get_default ();
-    if ( config.dpi > 0 ) {
-        pango_cairo_font_map_set_resolution ( (PangoCairoFontMap *) font_map, (double) config.dpi );
-    }
-    p_context = pango_font_map_create_context ( font_map );
+void textbox_set_pango_context ( PangoContext *p )
+{
+    textbox_cleanup ();
+    p_context = g_object_ref ( p );
+    p_metrics = pango_context_get_metrics ( p_context, NULL, NULL );
 }
 
 void textbox_cleanup ( void )
 {
+    if ( p_metrics ) {
+        pango_font_metrics_unref ( p_metrics );
+        p_metrics = NULL;
+    }
     if ( p_context ) {
         g_object_unref ( p_context );
         p_context = NULL;
@@ -664,7 +707,8 @@ void textbox_cleanup ( void )
 
 int textbox_get_width ( textbox *tb )
 {
-    return textbox_get_font_width ( tb ) + 2 * SIDE_MARGIN;
+    unsigned int offset = ( tb->flags & TB_INDICATOR ) ? DOT_OFFSET : 0;
+    return textbox_get_font_width ( tb ) + 2 * SIDE_MARGIN + offset;
 }
 
 int textbox_get_height ( textbox *tb )
@@ -688,26 +732,12 @@ int textbox_get_font_width ( textbox *tb )
 
 double textbox_get_estimated_char_width ( void )
 {
-    PangoFontDescription *pfd = pango_font_description_from_string ( config.menu_font );
-    // Get width
-    PangoFontMetrics     *metric = pango_context_get_metrics ( p_context, pfd, NULL );
-    int                  width   = pango_font_metrics_get_approximate_char_width ( metric );
-    pango_font_metrics_unref ( metric );
-
-    pango_font_description_free ( pfd );
+    int width = pango_font_metrics_get_approximate_char_width ( p_metrics );
     return ( width ) / (double) PANGO_SCALE;
 }
 
 int textbox_get_estimated_char_height ( void )
 {
-    // Set font.
-    PangoFontDescription *pfd = pango_font_description_from_string ( config.menu_font );
-
-    // Get width
-    PangoFontMetrics *metric = pango_context_get_metrics ( p_context, pfd, NULL );
-    int              height  = pango_font_metrics_get_ascent ( metric ) + pango_font_metrics_get_descent ( metric );
-    pango_font_metrics_unref ( metric );
-
-    pango_font_description_free ( pfd );
+    int height = pango_font_metrics_get_ascent ( p_metrics ) + pango_font_metrics_get_descent ( p_metrics );
     return ( height ) / PANGO_SCALE + 2 * SIDE_MARGIN;
 }
